/*-----------------------------------------------------------------------+
| Doom Runner: A cross-platform GUI frontend for Doom.                   |
| Copyright (C) 2011 Nick Virag                                          |
+------------------------------------------------------------------------+
| This program is free software: you can redistribute it and/or modify   |
| it under the terms of the GNU General Public License as published by   |
| the Free Software Foundation, either version 3 of the License, or      |
| any later version.                                                     |
+------------------------------------------------------------------------+
| This program is distributed in the hope that it will be useful,        |
| but WITHOUT ANY WARRANTY; without even the implied warranty of         |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
| GNU General Public License for more details.                           |
+------------------------------------------------------------------------+
| You should have received a copy of the GNU General Public License      |
| along with this program.  If not, see <http://www.gnu.org/licenses/>.  |
+------------------------------------------------------------------------+
| Contact Nick Virag at: <nick@win31.net>, <support@retronuggetdev.com>, |
| or <nickel516@gmail.com>.                                              |
+-----------------------------------------------------------------------*/

#include <QtGui/QtGui>
#include <fstream>
#include <iostream>
#include <sstream>
#include <stdio.h>
#include <algorithm>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <vector>
#include "main.h"
#include "doomrunner.h"
#include "config.h"
#include "addnewfile.h"

string newiwadlistitem2;
string newgamenotes2;
int lastprofile = 1;

int getdir2 (string dir, vector<string> &files) {
    DIR *dp;
    struct dirent *dirp;
    if((dp  = opendir(dir.c_str())) == NULL) {
        cout << "Error(" << errno << ") opening " << dir << endl;
        return errno;
    }
    while ((dirp = readdir(dp)) != NULL) { files.push_back(string(dirp->d_name)); }
    closedir(dp);
    return 0;
}

addnewfile::addnewfile() {
    this->setFixedSize(430,110);
    QLabel *gamenamelabel = new QLabel(this);
    gamenamelabel->setText("Game name:");
    gamename = new QLineEdit(this);
    QLabel *gamenoteslabel = new QLabel(this);
    gamenoteslabel->setText("Game notes:");
    gamenotes = new QLineEdit(this);
    QPushButton *closeform = new QPushButton(this);
    closeform -> setText ( "Ok" );
    closeform -> setDefault ( true );
    QPushButton *cancelform = new QPushButton ( this );
    cancelform -> setText ( "Cancel" );
    connect ( cancelform, SIGNAL ( clicked () ), this, SLOT ( cancelclick () ) );
    connect(closeform, SIGNAL(clicked()), this, SLOT(okclick()));
    if (preadf[0] == "") QMessageBox::information(this, "Doom Runner", "This is your first running Doom Runner. You need to set up a profile before playing Doom.\nYou will be promted for some of the basic options.\nAddititonal settings can be accessed later using the Edit button.");
    displayswitch = 3;
    stringstream lastprofilestring;
    bool whiletoggle = true;
    bool writetofile = false;
    while (whiletoggle == true) {
        lastprofilestring.str(std::string());
        lastprofilestring << lastprofile;
        string selectedprofile = startdir + "/profiles/profile" + lastprofilestring.str() + ".txt";
        currentmenuitem = lastprofilestring . str ();
        ifstream profilecount(selectedprofile.c_str());
        if (profilecount) {
            lastprofile++;
            profilecount.close();
        }
        else {
            whiletoggle = false;
            string dir = string(".");
            vector<string> files = vector<string>();
            getdir2(startdir + "/wads/iwads",files);
            size_t filesid;
            string iwadlistitem;
            unsigned int filescount = 0;
            bool filesreturn = true;
            bool newiwadfile = true;
            while (filesreturn == true) {
                iwadlistitem = files[filescount];
                cout << iwadlistitem << "\n";
                transform(iwadlistitem.begin(), iwadlistitem.end(), iwadlistitem.begin(), ::tolower);
                filesid = iwadlistitem.find(".wad");
                if (filesid != string::npos) {
                    transform(iwadlistitem.begin(), iwadlistitem.begin()+1, iwadlistitem.begin(), ::toupper);
                    iwadlistitem.erase(iwadlistitem.end()-4,iwadlistitem.end());
                    writetofile = true;
                    filesreturn = false;
                    newiwadfile = false;
                }
                else {
                    newiwadfile = true;
                    filescount++;
                    if (filescount < files.size()) { filesreturn = true; }
                    else { filesreturn = false; }
                }
            }
            cout << iwadlistitem << "\n";
            if (newiwadfile == true) {
                bool whilereturn = true;
                QMessageBox::StandardButton ret;
                QMessageBox::StandardButton freedoom;
                while(whilereturn == true) {
                    ret = QMessageBox::question(this, "Doom Runner", "No primary wad (iwad) was found.\nDon't panic! This is likely your first time running Doom Runner. Click (Yes) to select an iwad that you have on your hard drive, or click (No) to get information on obtaining an iwad.", QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
                    if (ret == QMessageBox::Yes) {
                        QString filename = QFileDialog::getOpenFileName(this, tr("Open Doom File"), "/", tr("Doom Files (*.wad *.deh);;All files (*.*)"));
                        string::size_type filesid2;
                        if (filename != "") {
                            string openfilestring = filename.toStdString();
                            transform(openfilestring.begin(), openfilestring.end(), openfilestring.begin(), ::tolower);
                            filesid2 = openfilestring.find_last_of("/");
                            if (filesid2 != string::npos) {
                                openfilestring.erase(0, filesid2 + 1);
                                transform(openfilestring.begin(), openfilestring.begin()+1, openfilestring.begin(), ::toupper);
                                int length = strlen(openfilestring.c_str());
                                string filetype = openfilestring.substr(length - 4, length);
                                if (filetype == ".wad") {
                                    openfilestring.erase(openfilestring.end()-4,openfilestring.end());
                                    if (openfilestring == "Doom1" || openfilestring == "Doom" || openfilestring == "Doom2" || openfilestring == "Freedoom" || openfilestring == "Tnt" || openfilestring == "Plutonia") {
                                        iwadlistitem = openfilestring;
                                        string stdoutput = startdir + "/wads/iwads/" + openfilestring + filetype;
                                        const char* output = stdoutput.c_str();
                                        std::ifstream ifs(filename.toStdString().c_str(), std::ios::binary);
                                        std::ofstream ofs(output, std::ios::binary);
                                        ofs << ifs.rdbuf();
                                        writetofile = true;
                                    }
                                    else { QMessageBox::warning(this, "Doom Runner", "It doesn't appear as though " + QString::fromStdString(openfilestring) + " is a valid IWAD."); }
                                }
                            }
                        }
                        whilereturn = false;
                    }
                    else if (ret == QMessageBox::No) {
                        freedoom = QMessageBox::question(this, "Doom Runner", "You can download a free and legal Doom iwad at\nhttp://www.nongnu.org/freedoom/download.html\nDownload the Complete IWAD version. When you have downloaded it, unzip the file and click (Ok). You will then be asked to open it using the Open File menu.", QMessageBox::Ok | QMessageBox::Cancel);
                        if (freedoom == QMessageBox::Ok) {
                            QString filename = QFileDialog::getOpenFileName(this, tr("Open Doom File"), "/", tr("Doom Files (*.wad *.deh);;All files (*.*)"));
                            string::size_type filesid2;
                            if (filename != "") {
                                string openfilestring = filename.toStdString();
                                transform(openfilestring.begin(), openfilestring.end(), openfilestring.begin(), ::tolower);
                                filesid2 = openfilestring.find_last_of("/");
                                if (filesid2 != string::npos) {
                                    openfilestring.erase(0, filesid2 + 1);
                                    transform(openfilestring.begin(), openfilestring.begin()+1, openfilestring.begin(), ::toupper);
                                    int length = strlen(openfilestring.c_str());
                                    string filetype = openfilestring.substr(length - 4, length);
                                    if (filetype == ".wad") {
                                        openfilestring.erase(openfilestring.end()-4,openfilestring.end());
                                        if (openfilestring == "Doom1" || openfilestring == "Doom" || openfilestring == "Doom2" || openfilestring == "Freedoom" || openfilestring == "Tnt" || openfilestring == "Plutonia") {
                                            iwadlistitem = openfilestring;
                                            string stdoutput = startdir + "/wads/iwads/" + openfilestring + filetype;
                                            const char* output = stdoutput.c_str();
                                            std::ifstream ifs(filename.toStdString().c_str(), std::ios::binary);
                                            std::ofstream ofs(output, std::ios::binary);
                                            ofs << ifs.rdbuf();
                                            whilereturn = false;
                                            writetofile = true;
                                        }
                                        else { QMessageBox::warning(this, "Doom Runner", "It doesn't appear as though " + QString::fromStdString(openfilestring) + " is a valid IWAD."); }
                                    }
                                }
                            }
                            whilereturn = false;
                        }
                        else { whilereturn = false; }
                    }
                    else { whilereturn = false; }
                }
            }
            if (writetofile == true) {
                stringstream tempnumber;
                int tempnumberint = 1;
                tempnumber << tempnumberint;
                newopenfile = selectedprofile;
                newiwadlistitem = iwadlistitem;
                cout << selectedprofile << "\n";
            }
        }
    }
    switch ( windowstyle ) {
    case 0:
        if ( ostoggle2 == 2 ) {
            gamenamelabel->setGeometry( 10, 17, 160, 16 );
            gamename->setGeometry( 100, 13, 315, 22 );
            gamenoteslabel->setGeometry( 10, 51, 160, 16 );
            gamenotes->setGeometry( 100, 48, 315, 22 );
            closeform -> setGeometry( 340, 75, 80, 32 );
            cancelform -> setGeometry ( 260, 75, 80, 32 );
            break;
        }
    default:
        gamenamelabel->setGeometry( 10, 17, 160, 16 );
        gamename->setGeometry( 100, 13, 315, 22 );
        gamenoteslabel->setGeometry( 10, 51, 160, 16 );
        gamenotes->setGeometry( 100, 48, 315, 22 );
        closeform->setGeometry( 335, 76, 80, 26 );
        cancelform -> setGeometry ( 245, 76, 80, 26 );
        break;
    }
    if ( sliderint == 1 ) { move ( QApplication::desktop()->width() / 2 - 215, QApplication::desktop()->height() / 2 - 55 ); }
    else { move ( windowx, windowy ); }
}

void addnewfile::okclick() {
    string line, type, executable, glexecutable, iwad, pwad1, pwad2, pwad3, pwad4, deh1, deh2, deh3, deh4, musictoggle, soundtoggle, opengltoggle, widthres, heightres, mousetoggle, joysticktoggle, addcmd;
    QString qnameexport = gamename->text();
    QString qnoteexport = gamenotes->text();
    if ( qnameexport == "" ) { QMessageBox::information( this, "Doom Runner", "Please enter the game's name." ); }
    else if ( qnameexport == "end" ) { QMessageBox::information( this, "Doom Runner", "You can't name a game that." ); }
    else {
        //if ( qnoteexport != "" ) {
        stringstream tempintloc;
        tempintloc.str(std::string());
        tempintloc << lastprofile - 1;
        string previousfile = startdir + "/profiles/profile" + tempintloc.str() + ".txt";
        ifstream previousfilecheck ( previousfile . c_str () );
        if ( previousfilecheck ) {
            string tinput;
            while ( ! previousfilecheck.eof() ) {
                getline ( previousfilecheck, line );
                tinput = tinput + "^" + line;
            }
            tinput.erase( 0, 1 );
            string::size_type parser = tinput.find_first_of( "^" );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            type = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            executable = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            glexecutable = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            iwad = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            pwad1 = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            pwad2 = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            pwad3 = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            pwad4 = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            deh1 = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            deh2 = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            deh3 = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            deh4 = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            musictoggle = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            soundtoggle = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            opengltoggle = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            widthres = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            heightres = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            mousetoggle = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            joysticktoggle = tinput.substr( 0, parser );
            tinput.erase( 0, parser + 1 );
            parser = tinput.find_first_of( "^" );
            addcmd = tinput.substr( 0, parser );
            previousfilecheck.close();
        }
            newgamename = qnameexport.toStdString();
            newgamenotes = qnoteexport.toStdString();
            ofstream openfile;
            openfile.open( newopenfile.c_str() );
            cout << newopenfile << "\n";
            openfile << newgamename << "\n" << newgamenotes << "\ndefault\n" << type << "\n" << executable << "\n" << glexecutable << "\n" << iwad << "\n" << pwad1 << "\n" << pwad2 << "\n" << pwad3 << "\n" << pwad4 << "\n" << deh1 << "\n" << deh2 << "\n" << deh3 << "\n" << deh4 << "\n" << musictoggle << "\n" << soundtoggle << "\n" << opengltoggle << "\n" << widthres << "\n" << heightres << "\n" << mousetoggle << "\n" << joysticktoggle << "\n" << addcmd;
            openfile.close();
            if ( sliderint != 1 ) {
                windowx = x();
                windowy = y();
            }
            titletransfer = newgamename;
            notestransfer = newgamenotes;
            imagetransfer = "default";
            this->close();
            interfaceviews( false );
            editprofile *widget = new editprofile;
            widget->show();
        //}
        //else { QMessageBox::information( this, "Doom Runner", "Please enter some notes." ); }
    }
}

void addnewfile::cancelclick () {
    if ( sliderint != 1 ) {
        windowx = x();
        windowy = y();
    }
    close();
    interfaceviews( false );
    entrywindow *widget = new entrywindow;
    widget->show();
}