/*-----------------------------------------------------------------------+
| Doom Runner: A cross-platform GUI frontend for Doom.                   |
| Copyright (C) 2011 Nick Virag                                          |
+------------------------------------------------------------------------+
| This program is free software: you can redistribute it and/or modify   |
| it under the terms of the GNU General Public License as published by   |
| the Free Software Foundation, either version 3 of the License, or      |
| any later version.                                                     |
+------------------------------------------------------------------------+
| This program is distributed in the hope that it will be useful,        |
| but WITHOUT ANY WARRANTY; without even the implied warranty of         |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
| GNU General Public License for more details.                           |
+------------------------------------------------------------------------+
| You should have received a copy of the GNU General Public License      |
| along with this program.  If not, see <http://www.gnu.org/licenses/>.  |
+------------------------------------------------------------------------+
| Contact Nick Virag at: <nick@win31.net>, <support@retronuggetdev.com>, |
| or <nickel516@gmail.com>.                                              |
+-----------------------------------------------------------------------*/
#include <QtGui/QtGui>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <vector>
#include <string>
#include <iostream>
#include <algorithm>
#include <fstream>
#include <sstream>
#include <ctime>
#include "config.h"
#include "doomrunner.h"
#include "main.h"
#include "addnewfile.h"

string prboomdir, prboomgldir, zdoomdir, zdoomgldir, legacydir, otherdir, othergldir;

int getdir ( string dir, vector<string> &files ) {
    DIR *dp;
    struct dirent *dirp;
    if( (dp  = opendir( dir.c_str() ) ) == NULL ) {
        cout << "Error(" << errno << ") opening " << dir << endl;
        return errno;
    }
    while ( ( dirp = readdir( dp ) ) != NULL ) { files.push_back( string ( dirp -> d_name ) ); }
    closedir( dp );
    return 0;
}

editprofile::editprofile() {
    editgame = new QWidget( this );
    QLabel *profilelabel = new QLabel( editgame );
    profilelabel -> setText( "Game name: " );
    profiletitle = new QLineEdit( editgame );
    profiletitle -> setText(QString::fromStdString( titletransfer ) );
    QLabel *profilenotes = new QLabel( editgame );
    profilenotes -> setText( "Game notes: " );
    profilenotesedit = new QLineEdit( editgame );
    profilenotesedit -> setText( QString::fromStdString( notestransfer ) );
    radio1 = new QRadioButton( "Use default image", editgame );
    radio2 = new QRadioButton( "Set custom image",editgame);
    customimageadd = new QPushButton( "...", editgame );
    customimageadd -> setEnabled( false );
    profileimage = new QGraphicsView( editgame );
    profileimage -> setGeometry( 10, 70, 171, 151 );
    profileimage -> setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
    profileimage -> setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
    loadimage = new QGraphicsScene( profileimage );
    loadimage -> setSceneRect( profileimage -> rect() );
    profileimage -> setScene( loadimage );
    if ( imagetransfer != "default" ) {
        imagedata = QString::fromStdString( imagetransfer );
        customimageadd -> setEnabled( true );
        radio2 -> setChecked( true );
    }
    else {
        imagedata = QString::fromStdString( startdir ) + "/profiles/default.png";
        radio1 -> setChecked( true );
    }
    QPixmap imagefile( imagedata );
    QSize scaledSize = imagefile.size();
    scaledSize.scale(profileimage->size(), Qt::KeepAspectRatio);
    loadimage->addPixmap(imagefile.scaled(profileimage->size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
    monstertoggle = new QCheckBox( "No mouse", editgame );
    joysticktoggle = new QCheckBox( "No joystick", editgame );
    QWidget *editlevel = new QWidget( this );
    QFrame *iwadselectborder = new QFrame( editlevel );
    iwadselectborder -> setFrameStyle( QFrame::StyledPanel );
    QLabel *iwadselectlabel = new QLabel( iwadselectborder );
    iwadselectlabel -> setText( "Select main wad:" );
    iwadselect = new QComboBox( iwadselectborder );
    string dir = string( "." );
    vector<string> files = vector<string>();
    getdir( startdir + "/wads/iwads", files );
    size_t filesid;
    string iwadlistitem;
    for ( unsigned int i = 0; i < files.size(); i++ ) {
        iwadlistitem = files[i];
        transform(iwadlistitem.begin(), iwadlistitem.end(), iwadlistitem.begin(), ::tolower);
        filesid = iwadlistitem.find(".wad");
		if (filesid != string::npos) {
            transform(iwadlistitem.begin(), iwadlistitem.begin()+1, iwadlistitem.begin(), ::toupper);
            iwadlistitem.erase(iwadlistitem.end()-4,iwadlistitem.end());
            if ( iwadlistitem != "Prboom" ) { iwadselect -> addItem(QString::fromStdString(iwadlistitem)); }
        }
    }
    QPushButton *addwadfile = new QPushButton(editlevel);
    addwadfile -> setText("Add a file");
    QFrame *pwadselectborder = new QFrame(editlevel);
    pwadselectborder -> setFrameStyle(QFrame::StyledPanel);
    QLabel *pwadselectlabel = new QLabel(pwadselectborder);
    pwadselectlabel -> setText("Select addon files:");
    pwadselectlist = new QListWidget ( pwadselectborder );
    pwadselectlist -> sortItems (  );
    QListWidgetItem *pwaditems [ 64 ] = { new QListWidgetItem () };
    if ( currentmenuitem != "0" ) {
        string linef;
        string wadfiledir = startdir + "/profiles/profile" + currentmenuitem + "f.txt";
        ifstream wadfileloc ( wadfiledir . c_str () );
        if ( wadfileloc.is_open () ) {
            string tinputf [ 64 ];
            bool inputchecked;
            for ( int inputloc = 0 ; ! wadfileloc.eof () ; inputloc ++ ) {
                if (  inputloc != 65 ) {
                    getline ( wadfileloc , linef );
                    if ( linef != "" ) {
                        tinputf [ inputloc ] = linef;
                        if ( tinputf [ inputloc ] . substr ( 0 , 1 ) == "c" ) { inputchecked = true; }
                        else { inputchecked = false; }
                        tinputf [ inputloc ] . erase ( 0, 1 );
                        pwaditems [ inputloc ] = new QListWidgetItem ( QString::fromStdString ( tinputf [ inputloc ] ) );
                        pwaditems [ inputloc ] -> setFlags ( pwaditems [ inputloc ] -> flags () | Qt::ItemIsUserCheckable );
                        if ( inputchecked == true ) { pwaditems [ inputloc ] -> setCheckState ( Qt::Checked ); }
                        else { pwaditems [ inputloc ] -> setCheckState ( Qt::Unchecked ); }
                        pwadselectlist -> addItem ( pwaditems [ inputloc ] );
                    }
                }
            }
            wadfileloc . close ();
        }
        filemoveup = new QPushButton ( pwadselectborder );
        filemoveup -> setText ( "Move Up" );
        filemovedown = new QPushButton ( pwadselectborder );
        filemovedown -> setText ( "Move Down" );
        filedelete = new QPushButton ( pwadselectborder );
        filedelete -> setText ( "Remove" );
    }
    editav = new QWidget ( this );
    QFrame *editaudio = new QFrame ( editav );
    editaudio -> setFrameStyle ( QFrame::StyledPanel );
    editsm = new QCheckBox ( "Enable Music" , editaudio );
    editsm -> setChecked ( true );
    editm = new QCheckBox ( "Enable Sound" , editaudio );
    editm -> setChecked ( true );
    edits = new QRadioButton ( "8-bit Graphics" , editaudio );
    editno = new QRadioButton ( "OpenGL Graphics" , editaudio );
    editno -> setChecked ( true );
    QLabel *displaylabel = new QLabel(editaudio);
    displaylabel -> setText("Aspect Ratio:");
    displayselect = new QComboBox(editaudio);
    displayselect -> addItem("4:3");
    displayselect -> addItem("16:9");
    displayselect -> addItem("16:10");
    QLabel *resolutionlabel = new QLabel(editaudio);
    resolutionlabel -> setText("Display Resolution:");
    resolutionselect = new QComboBox(editaudio);
    QFrame *editvideo = new QFrame(editav);
    displayimage = new QGraphicsView(editvideo);
    displayimage -> setGeometry(21,15,240,180);
    editvideo -> setFrameStyle(QFrame::StyledPanel);
    displayimage -> setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    displayimage -> setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    loaddisplayimage = new QGraphicsScene ( displayimage );
    loaddisplayimage -> setSceneRect ( displayimage -> rect () );
    displayimage -> setScene ( loaddisplayimage );
    advancededit = new QWidget ( this );
    QLabel *cmdparams = new QLabel (advancededit);
    cmdparams -> setText ("Additional commands:");
    cmdparamsedit = new QLineEdit ( advancededit );
    QLabel *newportlabel = new QLabel ( advancededit );
    newportlabel -> setText ( "Select source port:" );
    newportselect = new QComboBox ( advancededit );
    newportselect -> addItem ( "Prboom (onboard)" );
    newportselect -> addItem ( "Prboom" );
    newportselect -> addItem ( "Prboom GL" );
    newportselect -> addItem ( "Prboom + GL" );
    newportselect -> addItem ( "ZDoom" );
    newportselect -> addItem ( "GZDoom" );
    newportselect -> addItem ( "ZDoom + GZDoom" );
    newportselect -> addItem ( "Doom Legacy" );
    newportselect -> addItem ( "Other..." );
    newportselect -> addItem ( "Other... + GL" );
    string line2;
    string spdir = startdir + "/profiles/profilesp.txt";
    ifstream spdirfile ( spdir . c_str () );
    if ( spdirfile . is_open () ) {
        string tinput;
        while ( ! spdirfile . eof () ) {
            getline ( spdirfile , line2 );
            tinput = tinput + "^" + line2;
	}
        tinput . erase ( 0 , 1 );
        string::size_type parser = tinput.find_first_of("^");
        prboomdir = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        prboomgldir = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        zdoomdir = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        zdoomgldir = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        legacydir = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        otherdir = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        othergldir = tinput.substr(0, parser);
        spdirfile.close();
    }
    regexe = new QPushButton ("...", advancededit);
    glexe = new QPushButton ("GL", advancededit);
    qlexecutable = new QLabel ( advancededit );
    qlexecutable -> setText ( "Path: " );
    qlglexecutable = new QLabel ( advancededit );
    qlglexecutable -> setText ( "GL Path: " );
    QLabel *aboutdoomrunner = new QLabel ( advancededit );
    aboutdoomrunner -> setWordWrap ( true );
    aboutdoomrunner -> setFrameShape ( QFrame::StyledPanel );
    aboutdoomrunner -> setAlignment ( Qt::AlignTop );
    aboutdoomrunner -> setText ( "Doom Runner 1.0.3 Copyright (C) 2011 Nicholas Virag\nThis program comes with ABSOLUTELY NO WARRANTY; This is free software, and you are welcome to redistribute it under certain conditions; for details see the included \"./gpl-3.0.txt\".\nContact: support@retronuggetdev.com\nWebsite: http://www.retronuggetdev.com" );
    editprofile::exebuttonselect ();
    doomrunneredit = new QWidget ( this );
    styleselect = new QComboBox ( doomrunneredit );
    styleselect -> addItem ( "Native Style" );
    styleselect -> addItem ( "Cleanlooks" );
    styleselect -> addItem ( "Plastique" );
    styleselect -> addItem ( "Qt Windows" );
    styleselect -> addItem ( "CDE" );
    styleselect -> addItem ( "Motif" );
    styleselect -> setCurrentIndex ( windowstyle );
    styleselecttext = new QLabel ( doomrunneredit );
    styleselecttext -> setText ( "Select style:" );
    styleselectdescription = new QLabel ( doomrunneredit );
    styleselectdescription -> setWordWrap ( true );
    styleselectdescription -> setFrameShape ( QFrame::StyledPanel );
    styleselectdescription -> setAlignment ( Qt::AlignTop );
    slideeffecttoggle = new QCheckBox ( doomrunneredit );
    slideeffecttoggle -> setText ( "Window Slide Effect" );
    if ( sliderint == 1 ) { slideeffecttoggle -> setChecked ( true ); }
    else { slideeffecttoggle -> setChecked ( false ); }
    if ( ostoggle2 == 1 ) { slideeffecttoggle -> setEnabled ( false ); }
    slideeffectvalue = new QSlider ( Qt::Horizontal, doomrunneredit );
    slideeffectvalue -> setValue ( sliderspeedint );
    slideeffecttext = new QLabel ( doomrunneredit );
    slideeffecttext -> setText ( "Effect Speed:" );
    nativemenubar = new QCheckBox ( doomrunneredit );
    if ( toolbartoggleint == 1 ) { nativemenubar -> setChecked ( true ); }
    else { nativemenubar -> setChecked ( false ); }
    nativemenubar -> setText ( "Native Style Menu Bar (Mac Only)" );
    setFixedSize ( 500, 310 );
    editnavbar = new QTabWidget ( this );
    if ( selectedtab < 4 ) {
        editnavbar -> addTab ( editgame, QString ( "General" ) );
        editnavbar -> addTab ( editlevel, QString ( "Levels" ) );
        editnavbar -> addTab ( editav, QString ( "Audio/Video" ) );
        editnavbar -> addTab ( advancededit, QString ( "Advanced" ) );
        doomrunneredit->setVisible( false );
        doomrunneredit->setEnabled( false );
        setWindowTitle ( "Edit " + QString::fromStdString ( titletransfer ) );
    }
    else {
        editgame -> setVisible ( false );
        editgame -> setEnabled ( false );
        editlevel -> setVisible ( false );
        editlevel -> setEnabled ( false );
        editav -> setVisible ( false );
        editav -> setEnabled ( false );
        advancededit -> setVisible ( false );
        advancededit -> setEnabled ( false );
        editnavbar -> addTab ( doomrunneredit, QString ( "DoomRunner" ) );
        setWindowTitle ( "DoomRunner Settings" );
    }
    QPushButton *buttonok = new QPushButton ( "OK", this );
    QPushButton *buttoncancel = new QPushButton ( "Cancel", this );
    buttonok -> setDefault ( true );
    QPushButton *buttonhelp = new QPushButton ( "Help", this );
    buttonautoset = new QPushButton ( "Auto", this );
	removeiwad = new QPushButton ( "Erase Iwad" , this );
    switch ( windowstyle ) {
        case 0:
            if ( ostoggle2 == 2 ) {
            	profilelabel -> setGeometry ( 10, 10, 90, 16 );
	            profiletitle -> setGeometry ( 100, 6, 360, 22 );
    	        profilenotes -> setGeometry ( 10, 40, 90, 16 );
    	        profilenotesedit -> setGeometry ( 100, 36, 360, 22 );
    	        radio1 -> setGeometry ( 190, 70, 140, 22 );
    	        radio2 -> setGeometry ( 190, 93 ,140, 22 );
    	        customimageadd -> setGeometry ( 325, 90, 57, 32 );
    	        monstertoggle -> setGeometry ( 190, 140, 140, 16 );
    	        joysticktoggle -> setGeometry ( 190, 163, 140, 16 );
    	        iwadselectborder -> setGeometry ( 10, 10, 300, 40 );
    	        iwadselectlabel -> setGeometry ( 10, 10, 150, 16 );
    	        iwadselect -> setGeometry ( 145, 7, 140, 26 );
    	        addwadfile -> setGeometry ( 315, 6, 156, 52 );
    	        pwadselectlabel -> setGeometry ( 10, 10, 150, 16 );
    	        pwadselectborder -> setGeometry( 10, 61, 455, 165 );
    	        editaudio -> setGeometry ( 10, 10, 160, 210 );
    	        editsm -> setGeometry ( 10, 10, 140, 22 );
    	        editm -> setGeometry ( 10, 33, 140, 22 );
    	        edits -> setGeometry ( 10, 56, 140, 22 );
    	        editno -> setGeometry ( 10, 79, 140, 22 );
    	        displaylabel -> setGeometry ( 10, 102, 140, 22 );
    	        displayselect -> setGeometry ( 10, 125, 140, 26 );
    	        resolutionlabel -> setGeometry ( 10, 155, 140, 22 );
    	        resolutionselect -> setGeometry ( 10, 176, 140, 26 );
    	        editvideo -> setGeometry ( 182, 10, 282, 210 );
    	        cmdparams -> setGeometry ( 10, 10, 140, 16 );
    	        cmdparamsedit -> setGeometry ( 160, 8, 300, 22 );
    	        newportlabel -> setGeometry ( 10, 38, 140, 16 );
    	        newportselect -> setGeometry ( 155, 36, 190, 26 );
    	        regexe -> setGeometry ( 350, 36, 60, 32 );
    	        glexe -> setGeometry ( 410, 36, 60, 32 );
    	        qlexecutable -> setGeometry ( 10,65,450,16 );
    	        qlglexecutable -> setGeometry ( 10,85,450,16 );
    	        editnavbar -> setGeometry ( 10, 10, 480, 260 );
    	        buttonok -> setGeometry ( 416, 275, 80, 32 );
    	        buttoncancel -> setGeometry ( 338, 275, 80, 32 );
    	        buttonhelp -> setGeometry ( 10, 275, 80, 32 );
    	        buttonautoset -> setGeometry ( 88, 275, 80, 32 );
				removeiwad -> setGeometry ( 88, 275, 100, 32 );
    	        aboutdoomrunner -> setGeometry ( 10, 110, 455, 115 );
    	        styleselect -> setGeometry ( 190, 7, 180, 26 );
    	        styleselecttext -> setGeometry ( 100, 10, 140, 22 );
    	        styleselectdescription -> setGeometry ( 10, 45, 455, 100 );
    	        slideeffecttoggle -> setGeometry ( 170, 155, 150, 16 );
    	        slideeffecttext -> setGeometry ( 10, 180, 150, 16 );
    	        slideeffectvalue -> setGeometry ( 110, 180, 355, 16 );
    	        nativemenubar -> setGeometry ( 130, 205, 240, 16 );
    	        pwadselectlist -> setGeometry ( 145 , 10 , 300 , 145 );
    	        filemoveup -> setGeometry ( 10 , 30, 125 , 32 );
    	        filemovedown -> setGeometry ( 10 , 65 , 125 , 32 );
    	        filedelete -> setGeometry ( 10 , 100 , 125 , 32 );
            	break;
            }
        default:
            profilelabel -> setGeometry( 10, 10, 90, 16 );
            profiletitle -> setGeometry( 100, 6, 360, 22 );
            profilenotes -> setGeometry( 10, 40, 90, 16 );
            profilenotesedit -> setGeometry( 100, 36, 360, 22 );
            radio1 -> setGeometry( 190, 70, 140, 22 );
            radio2 -> setGeometry( 190, 93 ,140, 22 );
            customimageadd -> setGeometry( 326, 92, 40, 26 );
            monstertoggle -> setGeometry( 190, 140, 140, 16 );
            joysticktoggle -> setGeometry( 190, 163, 140, 16 );
            iwadselectborder -> setGeometry( 10, 10, 300, 40 );
            iwadselectlabel -> setGeometry( 10, 10, 150, 16 );
            iwadselect -> setGeometry( 145, 7, 140, 26 );
            addwadfile -> setGeometry( 320, 9, 143, 43 );
            pwadselectlabel -> setGeometry( 10, 10, 150, 16 );
            pwadselectborder -> setGeometry( 10, 61, 455, 165 );
            editaudio -> setGeometry( 10, 10, 160, 210 );
            editsm -> setGeometry( 10, 10, 140, 22 );
            editm -> setGeometry( 10, 33, 140, 22 );
            edits -> setGeometry( 10, 56, 140, 22 );
            editno -> setGeometry( 10, 79, 140, 22 );
            displaylabel -> setGeometry( 10, 102, 140, 22 );
            displayselect -> setGeometry( 10, 125, 140, 26 );
            resolutionlabel -> setGeometry( 10, 155, 140, 22 );
            resolutionselect -> setGeometry( 10, 176, 140, 26 );
            editvideo -> setGeometry( 182, 10, 282, 210 );
            cmdparams -> setGeometry( 10, 10, 140, 16 );
            cmdparamsedit -> setGeometry( 160, 8, 305, 22 );
            newportlabel -> setGeometry( 10, 38, 140, 16 );
            newportselect -> setGeometry( 160, 36, 185, 26 );
            regexe -> setGeometry( 355, 36, 50, 26 );
            glexe -> setGeometry( 415, 36, 50, 26 );
            qlexecutable -> setGeometry( 10,65,450,16 );
            qlglexecutable -> setGeometry( 10,85,450,16 );
            editnavbar -> setGeometry( 10, 10, 480, 260 );
            buttonok -> setGeometry( 410, 277, 80, 26 );
            buttoncancel -> setGeometry( 322, 277, 80, 26 );
            buttonhelp -> setGeometry( 10, 277, 80, 26 );
            buttonautoset -> setGeometry( 98, 277, 80, 26 );
			removeiwad -> setGeometry( 98, 277, 80, 26 );
            aboutdoomrunner -> setGeometry( 10, 110, 455, 115 );
            styleselect -> setGeometry ( 190, 7, 180, 26 );
            styleselecttext -> setGeometry ( 100, 10, 140, 22 );
            styleselectdescription -> setGeometry ( 10, 45, 455, 100 );
            slideeffecttoggle -> setGeometry ( 170, 155, 150, 16 );
            slideeffecttext -> setGeometry ( 10, 180, 150, 16 );
            slideeffectvalue -> setGeometry ( 110, 180, 355, 16 );
            nativemenubar -> setGeometry ( 130, 200, 240, 16 );
            nativemenubar -> setEnabled ( false );
            pwadselectlist -> setGeometry ( 145 , 10 , 300 , 145 );
            filemoveup -> setGeometry ( 10 , 30, 125 , 26 );
            filemovedown -> setGeometry ( 10 , 65 , 125 , 26 );
            filedelete -> setGeometry ( 10 , 100 , 125 , 26 );
            break;
    }
    buttonautoset -> hide();
	removeiwad -> hide ();
    connect ( addwadfile, SIGNAL ( clicked () ), this, SLOT ( addfilebutton () ) );
    connect ( radio1, SIGNAL ( clicked () ), this, SLOT ( radioselect1 () ) );
    connect ( radio2, SIGNAL ( clicked () ), this, SLOT ( radioselect1 () ) );
    connect ( customimageadd, SIGNAL( clicked() ), this, SLOT( customimageaddbutton() ) );
    connect ( buttoncancel, SIGNAL(clicked()), this, SLOT(cancelbutton()));
    connect ( buttonok, SIGNAL(clicked()), this, SLOT(writeprofile()));
    connect ( displayselect, SIGNAL(currentIndexChanged(int)), this, SLOT(changeresolutionselect()) );
    connect ( edits, SIGNAL(clicked()), this, SLOT(bitselected()));
    connect ( editno, SIGNAL(clicked()), this, SLOT(glselected()));
    connect ( editnavbar, SIGNAL(currentChanged(int)), this, SLOT(navbarselect()));
    connect ( buttonautoset, SIGNAL(clicked()), this, SLOT(autobuttondisplay()));
    connect ( newportselect, SIGNAL( currentIndexChanged( int ) ), this, SLOT(exebuttonselect() ) );
    connect ( regexe, SIGNAL( clicked() ), this, SLOT( fileselect() ) );
    connect ( glexe, SIGNAL( clicked() ), this, SLOT ( glfileselect() ) );
    connect ( styleselect, SIGNAL ( currentIndexChanged ( int ) ), this, SLOT ( styleselectslot () ) );
    connect ( slideeffecttoggle, SIGNAL ( stateChanged ( int ) ), this, SLOT ( slideeffectshowhide () ) );
    connect ( buttonhelp, SIGNAL ( clicked () ), this, SLOT ( buttonhelpclicked () ) );
    connect ( filemoveup , SIGNAL ( clicked () ) , this , SLOT ( buttonmoveup () ) );
    connect ( filemovedown , SIGNAL ( clicked () ) , this , SLOT ( buttonmovedown () ) );
    connect ( filedelete , SIGNAL ( clicked () ) , this , SLOT ( buttonremove () ) );
	connect ( removeiwad , SIGNAL ( clicked () ) , this , SLOT ( buttonremoveiwad () ) );
    string type, executable, glexecutable, line, iwad, pwad1, pwad2, pwad3, pwad4, deh1, deh2, deh3, deh4, musictoggle, soundtoggle, opengltoggle, widthres, heightres, mousetoggle, joysticktoggletxt, addcmd;
    int indexcount = 0;
    if (currentmenuitem != "0") {
    string settingsdir = startdir + "/profiles/profile" + currentmenuitem + ".txt";
    ifstream settingsfile (settingsdir.c_str());
    if (settingsfile.is_open()) {
        string tinput;
        while (! settingsfile.eof() ) {
            getline (settingsfile,line);
            tinput = tinput + "^" + line;
	}
        tinput.erase(0,1);
        string::size_type parser = tinput.find_first_of("^");
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        type = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        executable = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        glexecutable = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        iwad = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        pwad1 = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        pwad2 = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        pwad3 = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        pwad4 = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        deh1 = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        deh2 = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        deh3 = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        deh4 = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        musictoggle = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        soundtoggle = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        opengltoggle = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        widthres = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        heightres = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        mousetoggle = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        joysticktoggletxt = tinput.substr(0, parser);
        tinput.erase(0,parser + 1);
        parser = tinput.find_first_of("^");
        addcmd = tinput.substr(0, parser);
        settingsfile.close();
    }
    readwidth = atoi(widthres.c_str());
    readheight = atoi(heightres.c_str());
    }
    if (type == "def-prboom") { newportselect -> setCurrentIndex(0); }
    else if (type == "prboom") { newportselect -> setCurrentIndex(1); }
    else if (type == "prboom=gl") { newportselect -> setCurrentIndex(2); }
    else if (type == "prboom+=gl") { newportselect -> setCurrentIndex(3); }
    else if (type == "zdoom") { newportselect -> setCurrentIndex(4); }
    else if (type == "zdoom=gl") { newportselect -> setCurrentIndex(5); }
    else if (type == "zdoom+=gl") { newportselect -> setCurrentIndex(6); }
    else if (type == "legacy") { newportselect -> setCurrentIndex(7); }
    else if (type == "other") { newportselect -> setCurrentIndex(8); }
    else if (type == "other+=gl") { newportselect -> setCurrentIndex(9); }
    if ( newportselect -> currentIndex() == 1 || newportselect -> currentIndex() == 3 || newportselect -> currentIndex() == 4 || newportselect -> currentIndex() == 6 || newportselect -> currentIndex() == 7 || newportselect -> currentIndex() == 8 || newportselect -> currentIndex() == 9 ) { qlexecutable -> setText( "Path: " + QString::fromStdString( executable ) ); }
    else { qlexecutable -> setText( "Path: invalid" ); }
    if ( newportselect -> currentIndex() == 2 || newportselect -> currentIndex() == 3 || newportselect -> currentIndex() == 5 || newportselect -> currentIndex() == 6 || newportselect -> currentIndex() == 9 ) { qlglexecutable -> setText( "GL Path: " + QString::fromStdString( glexecutable ) ); }
    else { qlglexecutable -> setText( "GL Path: invalid" ); }
    while (iwadselect -> count() != indexcount) {
            iwadselect -> setCurrentIndex(indexcount);
            indexcount += 1;
            if (iwadselect -> currentText() == QString::fromStdString(iwad)) { indexcount = iwadselect -> count(); }
        }
        if (musictoggle == "<nomusic>") { editsm -> setChecked(false); }
        if (soundtoggle == "<nosound>") { editm -> setChecked(false); }
        if (opengltoggle == "true") { editno -> setChecked(true); }
        else { edits -> setChecked(true); }
        if (mousetoggle == "<nomouse>") { monstertoggle -> setChecked ( true ); }
        else { monstertoggle -> setChecked ( false ); }
        if (joysticktoggletxt == "<nojoystick>") { joysticktoggle -> setChecked ( true ); }
        if ( addcmd != "<none>" ) { cmdparamsedit -> setText( QString::fromStdString ( addcmd ) ); }
    if ( currentmenuitem != "0" ) {
        restoggle = true;
        editprofile::changeresolutionselect ();
        restoggle = false;
    }
    editprofile::styleselectslot ();
    editprofile::slideeffectshowhide ();
    show ();
    editprofile::moveconfigwindow ();
}

void editprofile::addfilebutton() {
    if (currentmenuitem != "0") {
        QString filename = QFileDialog::getOpenFileName(this, tr("Open Doom File"), "/", tr("Doom Files (*.wad *.deh);;All files (*.*)"));
        string::size_type filesid;
        if (filename != "") {
            string openfilestring = filename.toStdString();
            transform(openfilestring.begin(), openfilestring.end(), openfilestring.begin(), ::tolower);
            filesid = openfilestring.find_last_of("/");
            if (filesid != string::npos) {
                openfilestring.erase(0, filesid + 1);
                transform(openfilestring.begin(), openfilestring.begin()+1, openfilestring.begin(), ::toupper);
                int length = strlen(openfilestring.c_str());
                string filetype = openfilestring.substr(length - 4, length);
                bool copytoggle = true;
                if (filetype == ".wad") {
                    openfilestring.erase(openfilestring.end()-4,openfilestring.end());
                    if (openfilestring == "Doom1" || openfilestring == "Doom" || openfilestring == "Doom2" || openfilestring == "Freedoom" || openfilestring == "Tnt" || openfilestring == "Plutonia" || openfilestring == "Chex" || openfilestring == "Chex2" || openfilestring == "Chex3" || openfilestring == "Doomu" || openfilestring == "Heretic" || openfilestring == "Hexdd" || openfilestring == "Hexen" || openfilestring == "Strife1" ) {
              		    iwadselect -> addItem(QString::fromStdString(openfilestring));
               	        string stdoutput = startdir + "/wads/iwads/" + openfilestring + filetype;
               	        const char* output = stdoutput.c_str();
               	        std::ifstream ifs(filename.toStdString().c_str(), std::ios::binary);
               	        std::ofstream ofs(output, std::ios::binary);
               	        ofs << ifs.rdbuf();
               	        copytoggle = false;
               	    }
				}
               	if ( copytoggle == true ) {
                   	QListWidgetItem *fileadd = new QListWidgetItem ( filename );
                   	fileadd -> setFlags ( fileadd -> flags () | Qt::ItemIsUserCheckable );
                   	fileadd -> setCheckState ( Qt::Checked );
                   	pwadselectlist -> addItem ( fileadd );
               	}
            }
        }
    }
}

void editprofile::radioselect1() {
    QString imagedata;
    if (radio1 -> isChecked()) {
        imagedata = QString::fromStdString(startdir) + "/profiles/default.png";
        customimageadd -> setEnabled(false);
    }
    else {
        imagedata = QString::fromStdString(imagetransfer);
        customimageadd -> setEnabled(true);
    }
    QPixmap imagefile(imagedata);
    QSize scaledSize = imagefile.size();
    scaledSize.scale(profileimage->size(), Qt::KeepAspectRatio);
    loadimage->addPixmap(imagefile.scaled(profileimage->size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
}

void editprofile::customimageaddbutton() {
    filename = QFileDialog::getOpenFileName(this, tr("Open Image"), "/", tr("Image Files (*.png *.bmp *.jpg *.gif);;All files (*.*)"));
    if (filename != "") {
        QString imagedata = filename;
        QPixmap imagefile(imagedata);
        QSize scaledSize = imagefile.size();
        scaledSize.scale(profileimage->size(), Qt::KeepAspectRatio);
        loadimage->addPixmap(imagefile.scaled(profileimage->size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
        //loadimage -> addPixmap(imagefile);
        imagetransfer = filename.toStdString();
    }
}

void editprofile::changeresolutionselect() {
    editprofile::arrangeresolutionset();
    if (displayswitch < 5) {
        displayswitch = 10;
        if (restoggle == false) {
            readwidth = QApplication::desktop() -> width();
            readheight = QApplication::desktop() -> height();
        }
    switch (readwidth) {
        case 320:
            if (readheight == 240) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(1);
            }
            else if (readheight == 200) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(0);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 400:
            if (readheight == 300) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(2);
            }
            else if (readheight == 225) {
                displayselect -> setCurrentIndex(1);
                resolutionselect -> setCurrentIndex(0);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 512:
            if (readheight == 384) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(3);
            }
            else if (readheight == 288) {
                displayselect -> setCurrentIndex(1);
                resolutionselect -> setCurrentIndex(1);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 640:
            if (readheight == 480) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(5);
            }
            else if (readheight == 400) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(4);
            }
            else if (readheight == 360) {
                displayselect -> setCurrentIndex(1);
                resolutionselect -> setCurrentIndex(2);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 720:
            if (readheight == 576) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(6);
            }
            else if (readheight == 480) {
                displayselect -> setCurrentIndex(2);
                resolutionselect -> setCurrentIndex(0);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 764:
            if (readheight == 522) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(7);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 800:
            if (readheight == 600) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(8);
            }
            else if (readheight == 450) {
                displayselect -> setCurrentIndex(1);
                resolutionselect -> setCurrentIndex(3);
            }
            else if (readheight == 480) {
                displayselect -> setCurrentIndex(2);
                resolutionselect -> setCurrentIndex(1);
            }
            else if (readheight == 500) {
                displayselect -> setCurrentIndex(2);
                resolutionselect -> setCurrentIndex(2);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution." ); }
            break;
        case 854:
            if (readheight == 480) {
                displayselect -> setCurrentIndex(1);
                resolutionselect -> setCurrentIndex(4);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution." ); }
            break;
        case 1024:
            if (readheight == 768) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(9);
            }
            else if (readheight == 576) {
                displayselect -> setCurrentIndex(1);
                resolutionselect -> setCurrentIndex(5);
            }
            else if (readheight == 640) {
                displayselect -> setCurrentIndex(2);
                resolutionselect -> setCurrentIndex(3);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 1152:
            if (readheight == 864) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(10);
            }
            else if (readheight == 648) {
                displayselect -> setCurrentIndex(1);
                resolutionselect -> setCurrentIndex(6);
            }
            else if (readheight == 720) {
                displayselect -> setCurrentIndex(2);
                resolutionselect -> setCurrentIndex(4);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 1280:
            if (readheight == 960) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(11);
            }
            else if (readheight == 1024) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(12);
            }
            else if (readheight == 720) {
                displayselect -> setCurrentIndex(1);
                resolutionselect -> setCurrentIndex(7);
            }
            else if (readheight == 800) {
                displayselect -> setCurrentIndex(2);
                resolutionselect -> setCurrentIndex(6);
            }
            else if (readheight == 768) {
                displayselect -> setCurrentIndex(2);
                resolutionselect -> setCurrentIndex(5);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 1366:
            if (readheight == 768) {
                displayselect -> setCurrentIndex(1);
                resolutionselect -> setCurrentIndex(8);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 1400:
            if (readheight == 1050) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(13);
            }
            else if (readheight == 787) {
                displayselect -> setCurrentIndex(1);
                resolutionselect -> setCurrentIndex(9);
            }
            else if (readheight == 785) {
                displayselect -> setCurrentIndex(2);
                resolutionselect -> setCurrentIndex(7);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 1440:
            if (readheight == 900) {
                displayselect -> setCurrentIndex(2);
                resolutionselect -> setCurrentIndex(8);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 1600:
            if (readheight == 1200) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(14);
            }
            else if (readheight == 900) {
                displayselect -> setCurrentIndex(1);
                resolutionselect -> setCurrentIndex(10);
            }
            else if (readheight == 1000) {
                displayselect -> setCurrentIndex(2);
                resolutionselect -> setCurrentIndex(9);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 1680:
            if (readheight == 1050) {
                displayselect -> setCurrentIndex(2);
                resolutionselect -> setCurrentIndex(10);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 1920:
            if (readheight == 1440) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(15);
            }
            else if (readheight == 1080) {
                displayselect -> setCurrentIndex(1);
                resolutionselect -> setCurrentIndex(11);
            }
            else if (readheight == 1200) {
                displayselect -> setCurrentIndex(1);
                resolutionselect -> setCurrentIndex(11);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 2048:
            if (readheight == 1536) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(16);
            }
            else if (readheight == 1152) {
                displayselect -> setCurrentIndex(1);
                resolutionselect -> setCurrentIndex(12);
            }
            else if (readheight == 1280) {
                displayselect -> setCurrentIndex(2);
                resolutionselect -> setCurrentIndex(12);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 2560:
            if (readheight == 1920) {
                displayselect -> setCurrentIndex(0);
                resolutionselect -> setCurrentIndex(17);
            }
            else if (readheight == 1440) {
                displayselect -> setCurrentIndex(1);
                resolutionselect -> setCurrentIndex(13);
            }
            else if (readheight == 1600) {
                displayselect -> setCurrentIndex(2);
                resolutionselect -> setCurrentIndex(13);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        case 5120:
            if (readheight == 3200) {
                displayselect -> setCurrentIndex(2);
                resolutionselect -> setCurrentIndex(14);
            }
            else { QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution."); }
            break;
        default:
            QMessageBox::warning( this, "Doom Runner", "Doom Runner was unable to identify your computer's resolution.");
            break;
    }
}
}

void editprofile::bitselected() {
    if (displayselect -> currentText() == "4:3") {
        QPixmap imagefile(QString::fromStdString(startdir) + "/profiles/res/8b43.png");
        loaddisplayimage -> addPixmap(imagefile);
    }
    else if (displayselect -> currentText() == "16:9") {
        QPixmap imagefile(QString::fromStdString(startdir) + "/profiles/res/8b169.png");
        loaddisplayimage -> addPixmap(imagefile);
    }
    else if (displayselect -> currentText() == "16:10") {
        QPixmap imagefile(QString::fromStdString(startdir) + "/profiles/res/8b1610.png");
        loaddisplayimage -> addPixmap(imagefile);
    }
}

void editprofile::glselected() {
    if (displayselect -> currentText() == "4:3") {
        QPixmap imagefile(QString::fromStdString(startdir) + "/profiles/res/gl43.png");
        loaddisplayimage -> addPixmap(imagefile);
    }
    else if (displayselect -> currentText() == "16:9") {
        QPixmap imagefile(QString::fromStdString(startdir) + "/profiles/res/gl169.png");
        loaddisplayimage -> addPixmap(imagefile);
    }
    else if (displayselect -> currentText() == "16:10") {
        QPixmap imagefile(QString::fromStdString(startdir) + "/profiles/res/gl1610.png");
        loaddisplayimage -> addPixmap(imagefile);
    }
}

void editprofile::arrangeresolutionset() {
    if (displayselect -> currentText() == "4:3") {
        resolutionselect -> clear();
        resolutionselect -> addItem("320x200");
        resolutionselect -> addItem("320x240");
        resolutionselect -> addItem("400x300");
        resolutionselect -> addItem("512x384");
        resolutionselect -> addItem("640x400");
        resolutionselect -> addItem("640x480");
        resolutionselect -> addItem("720x576");
        resolutionselect -> addItem("764x522");
        resolutionselect -> addItem("800x600");
        resolutionselect -> addItem("1024x768");
        resolutionselect -> addItem("1152x864");
        resolutionselect -> addItem("1280x960");
        resolutionselect -> addItem("1280x1024");
        resolutionselect -> addItem("1400x1050");
        resolutionselect -> addItem("1600x1200");
        resolutionselect -> addItem("1920x1440");
        resolutionselect -> addItem("2048x1536");
        resolutionselect -> addItem("2560x1920");
        if (edits -> isChecked() == true) {
            QPixmap imagefile(QString::fromStdString(startdir) + "/profiles/res/8b43.png");
            loaddisplayimage -> addPixmap(imagefile);
        }
        else {
            QPixmap imagefile(QString::fromStdString(startdir) + "/profiles/res/gl43.png");
            loaddisplayimage -> addPixmap(imagefile);
        }
    }
    else if (displayselect -> currentText() == "16:9") {
        resolutionselect -> clear();
        resolutionselect -> addItem("400x225");
        resolutionselect -> addItem("512x288");
        resolutionselect -> addItem("640x360");
        resolutionselect -> addItem("800x450");
        resolutionselect -> addItem("854x480");
        resolutionselect -> addItem("1024x576");
        resolutionselect -> addItem("1152x648");
        resolutionselect -> addItem("1280x720");
        resolutionselect -> addItem("1366x768");
        resolutionselect -> addItem("1400x787");
        resolutionselect -> addItem("1600x900");
        resolutionselect -> addItem("1920x1080");
        resolutionselect -> addItem("2048x1152");
        resolutionselect -> addItem("2560x1440");
        if (edits -> isChecked() == true) {
            QPixmap imagefile(QString::fromStdString(startdir) + "/profiles/res/8b169.png");
            loaddisplayimage -> addPixmap(imagefile);
        }
        else {
            QPixmap imagefile(QString::fromStdString(startdir) + "/profiles/res/gl169.png");
            loaddisplayimage -> addPixmap(imagefile);
        }
    }
    else if (displayselect -> currentText() == "16:10") {
        resolutionselect -> clear();
        resolutionselect -> addItem("720x480");
        resolutionselect -> addItem("800x480");
        resolutionselect -> addItem("800x500");
        resolutionselect -> addItem("1024x600");
        resolutionselect -> addItem("1152x720");
        resolutionselect -> addItem("1280x768");
        resolutionselect -> addItem("1280x800");
        resolutionselect -> addItem("1400x875");
        resolutionselect -> addItem("1440x900");
        resolutionselect -> addItem("1600x1000");
        resolutionselect -> addItem("1680x1050");
        resolutionselect -> addItem("1920x1200");
        resolutionselect -> addItem("2048x1280");
        resolutionselect -> addItem("2560x1600");
        resolutionselect -> addItem("5120x3200");
        if (edits -> isChecked() == true) {
            QPixmap imagefile(QString::fromStdString(startdir) + "/profiles/res/8b1610.png");
            loaddisplayimage -> addPixmap(imagefile);
        }
        else {
            QPixmap imagefile(QString::fromStdString(startdir) + "/profiles/res/gl1610.png");
            loaddisplayimage -> addPixmap(imagefile);
        }
    }
}

void editprofile::navbarselect() {
    if ( editnavbar -> currentIndex () == 2 ) { buttonautoset -> show (); removeiwad -> hide ();}
    else if ( editnavbar -> currentIndex () == 1 ) { buttonautoset -> hide (); removeiwad -> show (); }
	else { buttonautoset -> hide (); removeiwad -> hide (); }
}

void editprofile::autobuttondisplay() {
    editno -> setChecked(true);
    displayswitch = 4;
    editprofile::changeresolutionselect();
}

void editprofile::exebuttonselect() {
    switch ( newportselect -> currentIndex() ) {
        case 0:
            qlexecutable -> setText( "Path: Invalid" );
            qlglexecutable -> setText( "GL Path: Invalid" );
            regexe -> setEnabled(false);
            glexe -> setEnabled(false);
            break;
        case 1:
            qlexecutable -> setText( "Path: " + QString::fromStdString ( prboomdir ) );
            qlglexecutable -> setText( "GL Path: Invalid" );
            regexe -> setEnabled(true);
            glexe -> setEnabled(false);
            break;
        case 2:
            qlexecutable -> setText( "Path: Invalid" );
            qlglexecutable -> setText( "GL Path: " + QString::fromStdString ( prboomgldir ) );
            regexe -> setEnabled(false);
            glexe -> setEnabled(true);
            break;
        case 3:
            qlexecutable -> setText( "Path: " + QString::fromStdString ( prboomdir ) );
            qlglexecutable -> setText( "GL Path: " + QString::fromStdString ( prboomgldir ) );
            regexe -> setEnabled(true);
            glexe -> setEnabled(true);
            break;
        case 4:
            qlexecutable -> setText( "Path: " + QString::fromStdString ( zdoomdir ) );
            qlglexecutable -> setText( "GL Path: Invalid" );
            regexe -> setEnabled(true);
            glexe -> setEnabled(false);
            break;
        case 5:
            qlexecutable -> setText( "Path: Invalid" );
            qlglexecutable -> setText( "GL Path: " + QString::fromStdString ( zdoomgldir ) );
            regexe -> setEnabled(false);
            glexe -> setEnabled(true);
            break;
        case 6:
            qlexecutable -> setText( "Path: " + QString::fromStdString ( zdoomdir ) );
            qlglexecutable -> setText( "GL Path: " + QString::fromStdString ( zdoomgldir ) );
            regexe -> setEnabled(true);
            glexe -> setEnabled(true);
            break;
        case 7:
            qlexecutable -> setText( "Path: " + QString::fromStdString ( legacydir ) );
            qlglexecutable -> setText( "GL Path: Invalid" );
            regexe -> setEnabled(true);
            glexe -> setEnabled(false);
            break;
        case 8:
            qlexecutable -> setText( "Path: " + QString::fromStdString ( otherdir ) );
            qlglexecutable -> setText( "GL Path: Invalid" );
            regexe -> setEnabled(true);
            glexe -> setEnabled(false);
            break;
        case 9:
            qlexecutable -> setText( "Path: " + QString::fromStdString ( otherdir ) );
            qlglexecutable -> setText( "GL Path: " + QString::fromStdString ( othergldir ) );
            regexe -> setEnabled(true);
            glexe -> setEnabled(true);
            break;
    }
}

void editprofile::writeprofile() {
    if ( qlexecutable -> text() == "Path: [None]" ) { QMessageBox::critical( this, "Doom Runner", "You need to select the executable path for your Doom source port!" ); }
    else if ( qlglexecutable -> text() == "GL Path: [None]" ) { QMessageBox::critical( this, "Doom Runner", "You need to select the OpenGL executable path for your Doom source port!" ); }
    else if ( profiletitle -> text() == "" ) { QMessageBox::critical( this, "Doom Runner", "You need to enter this game configuration's title!" ); }
    else {
        ofstream profilestream;
        string tempoutput;
        string openfile = startdir + "/profiles/profile" + currentmenuitem + ".txt";
        profilestream.open (openfile.c_str());
        profilestream.clear();
        profilestream << profiletitle -> text().toStdString() << "\n";
        profilestream << profilenotesedit -> text().toStdString() << "\n";
        tempoutput = startdir + "/profiles/default.png";
        if (radio1 -> isChecked() == true) { profilestream << "default\n"; }
        else {
            if (filename != "") { profilestream << filename.toStdString() << "\n"; }
            else { profilestream << imagetransfer << "\n"; }
        }
        string exeoutput = qlexecutable -> text().toStdString();
        string glexeoutput = qlglexecutable -> text().toStdString();
        exeoutput.erase( 0, 6 );
        glexeoutput.erase( 0, 9 );
        transform(exeoutput.begin(), exeoutput.end(), exeoutput.begin(), ::tolower);
        transform(glexeoutput.begin(), glexeoutput.end(), glexeoutput.begin(), ::tolower);
        if ( newportselect -> currentIndex() == 0 ) { profilestream << "def-prboom\ninvalid\ninvalid\n"; }
        else if ( newportselect -> currentIndex() == 1 ) { profilestream << "prboom\n" << exeoutput << "\ninvalid\n"; }
        else if ( newportselect -> currentIndex() == 2 ) { profilestream << "prboom=gl\ninvalid\n" << glexeoutput << "\n"; }
        else if ( newportselect -> currentIndex() == 3 ) { profilestream << "prboom+=gl\n" << exeoutput << "\n" << glexeoutput << "\n"; }
        else if ( newportselect -> currentIndex() == 4 ) { profilestream << "zdoom\n" << exeoutput << "\ninvalid\n"; }
        else if ( newportselect -> currentIndex() == 5 ) { profilestream << "zdoom=gl\ninvalid\n" << glexeoutput << "\n"; }
        else if ( newportselect -> currentIndex() == 6 ) { profilestream << "zdoom+=gl\n" << exeoutput << "\n" << glexeoutput << "\n"; }
        else if ( newportselect -> currentIndex() == 7 ) { profilestream << "legacy\n" << exeoutput << "\ninvalid\n"; }
        else if ( newportselect -> currentIndex() == 8 ) { profilestream << "other\n" << exeoutput << "\ninvalid\n"; }
        else if ( newportselect -> currentIndex() == 9 ) { profilestream << "other+=gl\n" << exeoutput << "\n" << glexeoutput << "\n"; }
        profilestream << iwadselect -> currentText().toStdString() << "\n<none>\n<none>\n<none>\n<none>\n<none>\n<none>\n<none>\n<none>\n";
        if (editsm -> isChecked () == true) { profilestream << "<music>\n"; }
        else { profilestream << "<nomusic>\n"; }
        if (editm -> isChecked () == true) { profilestream << "<sound>\n"; }
        else { profilestream << "<nosound>\n"; }
        if (edits -> isChecked () == true) { profilestream << "false\n"; }
        else { profilestream << "true\n"; }
        QString qtempoutput = resolutionselect -> currentText();
        tempoutput = qtempoutput.toStdString();
        string::size_type tempoutputint = tempoutput.find_first_of("x");
        string tempoutput2 = tempoutput;
        string outputwidth = tempoutput.erase(tempoutputint, tempoutput.size());
        string outputheight = tempoutput2.erase(0,tempoutputint + 1);
        profilestream << outputwidth << "\n";
        profilestream << outputheight << "\n";
        if (monstertoggle -> isChecked() == true) { profilestream << "<nomouse>\n"; }
        else { profilestream << "<mouse>\n"; }
        if (joysticktoggle -> isChecked() == true) { profilestream << "<nojoystick>\n"; }
        else { profilestream << "<joystick>\n"; }
        if (cmdparamsedit -> text() != "") { profilestream << cmdparamsedit -> text().toStdString() << "\n"; }
        else { profilestream << "<none>\n"; }
        profilestream.close();
        ofstream profilestreamsp;
        string openfilesp = startdir + "/profiles/profilesp.txt";
        profilestreamsp . open ( openfilesp . c_str () );
        profilestreamsp . clear ();
        profilestreamsp << prboomdir << "\n" << prboomgldir << "\n" << zdoomdir << "\n" << zdoomgldir << "\n" << legacydir << "\n" << otherdir << "\n" << othergldir;
        profilestreamsp . close ();
        bool tmpboolcheck = true;
        if ( toolbartoggleint == 0 ) { tmpboolcheck = false; }
        bool tmpboolcheck2 = true;
        if ( sliderint == 0 ) { tmpboolcheck2 = false; }
        ofstream wadfileloc;
        string wadfiledir = startdir + "/profiles/profile" + currentmenuitem + "f.txt";
        wadfileloc . open ( wadfiledir . c_str () );
        wadfileloc . clear ();
        if ( pwadselectlist -> count () != 0 ) {
            for ( int pwadselecta = 0 ; pwadselecta != pwadselectlist -> count () ; pwadselecta ++ ) {
                pwadselectlist -> setCurrentRow ( pwadselecta );
                QListWidgetItem *check = new QListWidgetItem ();
                check = pwadselectlist -> currentItem ();
                if ( check -> checkState () == Qt::Checked ) { wadfileloc << "c" << pwadselectlist -> currentItem () -> text () . toStdString () << "\n"; }
                else { wadfileloc << "u" << pwadselectlist -> currentItem () -> text () . toStdString () << "\n"; }
            }
        }
        if ( windowstyle != styleselect -> currentIndex () || nativemenubar -> isChecked () != tmpboolcheck || slideeffecttoggle -> isChecked () != tmpboolcheck2 || slideeffectvalue -> value () != sliderspeedint ) {
            openfile = startdir + "/profiles/profile0.txt";
            profilestream.open ( openfile.c_str () );
            profilestream.clear ();
            if ( nativemenubar -> isChecked () == false || nativemenubar -> isEnabled () == false ) { profilestream << "0\n"; }
            else { profilestream << "1\n"; }
            profilestream << styleselect -> currentIndex () << "\n";
            if ( slideeffecttoggle -> isChecked () == true && slideeffectvalue -> value () != 0 ) { profilestream << "1\n" << slideeffectvalue -> value () << "\n"; }
            else if ( slideeffecttoggle -> isChecked () == true ) { profilestream << "1\n1\n"; }
            else { profilestream << "0\n1\n"; }
			profilestream << glestoggle << "\n" << askruntoggle << "\n" << globaltoggle << "\n";
            profilestream.close ();
            QMessageBox::information( this, "Doom Runner", "Some of the changes will not take effect until DoomRunner is re-launched.");
        }
        if ( sliderint == 1 ) { for ( int i = QApplication::desktop() -> width() / 2 - 250; i > -500; i -= sliderspeedint ) { move( i, QApplication::desktop() -> height() / 2 - 155 ); } }
        else {
            windowx = x();
            windowy = y();
        }
        close ();
        interfaceviews(false);
        entrywindow *widget = new entrywindow;
        widget -> show();
    }
}

void editprofile::cancelbutton() {
    if ( sliderint == 1 ) { for ( int i = QApplication::desktop() -> width() / 2 - 250; i > -500; i -= sliderspeedint ) { move( i, QApplication::desktop() -> height() / 2 - 155 ); } }
    else {
        windowx = x();
        windowy = y();
    }
    close ();
    interfaceviews( false );
    entrywindow *widget = new entrywindow;
    widget -> show();
}


void editprofile::fileselect() {
    QString filename = QFileDialog::getOpenFileName( this, tr( "Select Doom Program" ), "/" );
    if ( filename != "" ) {
        stringstream tmpchar;
        string stexecutable = filename.toStdString();
        transform(stexecutable.begin(), stexecutable.end(), stexecutable.begin(), ::tolower);
        tmpchar << stexecutable.at( stexecutable.length() - 4 ) << stexecutable.at( stexecutable.length() - 3 ) << stexecutable.at( stexecutable.length() - 2 ) << stexecutable.at( stexecutable.length() - 1 );
        if ( tmpchar.str() == ".app" ) {
            switch ( newportselect -> currentIndex() ) {
                case 1:
                    stexecutable += "/Contents/MacOS/PrBoom";
                    break;
                case 2:
                    stexecutable += "/Contents/MacOS/PrBoom";
                    break;
                case 3:
                    stexecutable += "/Contents/MacOS/PrBoom";
                    break;
                case 4:
                    stexecutable += "/Contents/MacOS/zdoom";
                    break;
                case 5:
                    QMessageBox::warning( this, "Doom Runner", "This Doom Source port is untested with Doom Runner. Please be aware that it may not function properly.");
                    stexecutable += "/Contents/MacOS/gzdoom";
                    break;
                case 6:
                    QMessageBox::warning( this, "Doom Runner", "This Doom Source port is untested with Doom Runner. Please be aware that it may not function properly.");
                    stexecutable += "/Contents/MacOS/zdoom";
                    break;
                case 7:
                    stexecutable += "/Contents/MacOS/Doom Legacy";
                    break;
            }
        }
        switch ( newportselect -> currentIndex() ) {
            case 1:
            case 3:
                prboomdir = stexecutable;
                break;
            case 4:
            case 6:
                zdoomdir = stexecutable;
                break;
            case 7:
                legacydir = stexecutable;
                break;
            case 8:
            case 9:
                otherdir = stexecutable;
                break;
        }
        qlexecutable -> setText( "Path: " + QString::fromStdString( stexecutable ) );
    }
}

void editprofile::glfileselect() {
    QString filename = QFileDialog::getOpenFileName( this, tr( "Select Doom Program" ), "/" );
    if ( filename != "" ) {
        stringstream tmpchar;
        string stglexecutable = filename.toStdString();
        transform(stglexecutable.begin(), stglexecutable.end(), stglexecutable.begin(), ::tolower);
        tmpchar << stglexecutable.at( stglexecutable.length() - 4 ) << stglexecutable.at( stglexecutable.length() - 3 ) << stglexecutable.at( stglexecutable.length() - 2 ) << stglexecutable.at( stglexecutable.length() - 1 );
        if ( tmpchar.str() == ".app" ) {
            switch ( newportselect -> currentIndex() ) {
                case 1:
                    stglexecutable += "/Contents/MacOS/PrBoom";
                    break;
                case 2:
                    stglexecutable += "/Contents/MacOS/PrBoom";
                    break;
                case 3:
                    stglexecutable += "/Contents/MacOS/PrBoom";
                    break;
                case 4:
                    stglexecutable += "/Contents/MacOS/zdoom";
                    break;
                case 5:
                    QMessageBox::warning( this, "Doom Runner", "This Doom Source port is untested with Doom Runner. Please be aware that it may not function properly.");
                    stglexecutable += "/Contents/MacOS/gzdoom";
                    break;
                case 6:
                    QMessageBox::warning( this, "Doom Runner", "This Doom Source port is untested with Doom Runner. Please be aware that it may not function properly.");
                    stglexecutable += "/Contents/MacOS/zdoom";
                    break;
                case 7:
                    stglexecutable += "/Contents/MacOS/Doom Legacy";
                    break;
            }
        }
        switch ( newportselect -> currentIndex() ) {
            case 2:
            case 3:
                prboomgldir = stglexecutable;
                break;
            case 5:
            case 6:
                zdoomgldir = stglexecutable;
                break;
            case 9:
                othergldir = stglexecutable;
                break;
        }
        qlglexecutable -> setText( "GL Path: " + QString::fromStdString( stglexecutable ) );
    }
}

void editprofile::moveconfigwindow() {
    if ( sliderint == 1 ) { for ( int i = -500; i < QApplication::desktop() -> width() / 2 - 250; i += sliderspeedint ) { move( i, QApplication::desktop() -> height() / 2 - 155 ); } } else { move ( windowx, windowy ); } }

void editprofile::styleselectslot () {
    switch ( styleselect -> currentIndex () ) {
        case 0:
            styleselectdescription -> setText ( "Native style: This sets DoomRunner to use your system's default theme. For example, if you are using a Mac, this will make DoomRunner Mac style, etc. This mode may appear distorted on some Linux systems, so other options are provided." );
            if ( ostoggle2 == 2 ) { nativemenubar -> setEnabled ( true ); }
            break;
        case 1:
            styleselectdescription -> setText ( "Cleanlooks style: This is a sleek, modern style used primarily on Linux systems." );
            nativemenubar -> setEnabled ( false );
            break;
        case 2:
            styleselectdescription -> setText ( "Plastique style: This is another modern style somewhat similar to Cleanlooks." );
            nativemenubar -> setEnabled ( false );
            break;
        case 3:
            styleselectdescription -> setText ( "Qt Windows style: This is a \"Fake Windows\" style that gives the appearance of Windows on non-Windows systems. If you are on a Windows computer, set the style to \"Native Style\" instead." );
            nativemenubar -> setEnabled ( false );
            break;
        case 4:
            styleselectdescription -> setText ( "CDE style: An old-fashioned style that is sometimes seen on very old Linux systems." );
            nativemenubar -> setEnabled ( false );
            break;
        case 5:
            styleselectdescription -> setText ( "Motif style: An incredibly early Unix style." );
            nativemenubar -> setEnabled ( false );
            break;
    }
}

void editprofile::slideeffectshowhide () {
    slideeffectvalue -> setEnabled ( slideeffecttoggle -> isChecked () );
    slideeffecttext -> setEnabled ( slideeffecttoggle -> isChecked () );
}

void editprofile::buttonhelpclicked () {
    switch ( editnavbar -> currentIndex () ) {
        case 0:
            if ( styleselect -> isEnabled () == true ) { QMessageBox::information( this , "Doom Runner Help" , "<b>Select style</b>: Set DoomRunner's window style. This will change the way things like the colors and font.<br><b>Window slide effect</b>: An effect that causes the window to slide onto the screen from the left. It may be too fast or too slow on some computers, so there is an adjustment.<br><b>Effect speed</b>: The speed adjustment for the \"Window Slide Effect\".<br><b>Native Style Menu Bar (Mac Only)</b>: On Mac computers, this can change the menu bar at the top of the screen to a different style.<br><b>Help</b>: Show this window." ); }
            else { QMessageBox::information( this , "Doom Runner Help" , "<b>Game name</b>: The game's name.<br><b>Game notes</b>: The notes section, shown underneath the picture in DoomRunner's main window. Not required.<br><b>Use default image</b>: Use an included, generic image to represent this game.<br><b>Set custom image</b>: Set a custom image. Supports GIF, PNG, JPG, and BMP images.<br><b>...</b> : Select a custom image.<br><b>No mouse</b>: Disable the mouse when playing Doom.<br><b>No joystick</b>: Disable the joystick when playing Doom.<br><b>Help</b>: Show this window." ); }
            break;
        case 1:
            QMessageBox::information( this , "Doom Runner Help" , "<b>Select main wad</b>: Set the game's main wad (IWAD). You can have one per game.<br><b>Select addon files</b>: Set the game's addon files (PWAD, DEHACKED, etc.). You can have up to 64 combined. For most ports, PWADs are added via the -file command and all other files are added via the -deh command. For Doom Legacy, all files are combined into the -file command.<br><b>Add a file</b>: Add a file to the DoomRunner options. It will automatically be sorted as a main wad or addon file.<br><b>Move Up</b>: Move a file on the list so that it is run sooner in sequence.<br><b>Move Down</b>: Move a file on the list so that it is run later in sequence.<br><b>Remove</b>: Removes a file from being run. This does not delete the file.<br><b>Help</b>: Show this window." );
            break;
        case 2:
            QMessageBox::information( this , "Doom Runner Help" , "<b>Enable Music</b>: Enable music.<br><b>Enable Sound</b>: Enable sound effects.<br><b>8-bit Graphics</b>: Use classic, original graphics.<br><b>OpenGL Graphics</b>: Use newer, better graphics. Some Doom source ports don't support this.<br><b>Aspect Ratio</b>: Set the aspect ratio for Doom. If you don't know what this is, don't worry, just press the \"Auto\" button next to the \"Help\" button when you have closed this window.<br><b>Display Resolution</b>: Set the display resolution for Doom. If you don't know what this is, don't worry, just press the \"Auto\" button next to the \"Help\" button when you have closed this window.<br><b>Auto</b>: Automatically set the graphics, aspect ratio, and display resolution.<br><b>Help</b>: Show this window." );
            break;
        case 3:
            QMessageBox::information( this , "Doom Runner Help" , "<b>Additional commands</b>: Add command-line options to Doom when it is run.<br><b>Select source port</b>: Select the source port to play Doom.<br><b>...</b> : Select the path to a non-OpenGL source port, such as ZDoom or Vanilla Doom.<br><b>GL</b>: Select the path to an OpenGL source port, such as GLBoom or GZDoom.<br><b>Help</b>: Show this window." );
            break;
    }
}

void editprofile::buttonmoveup () {
    if ( pwadselectlist -> currentRow () > 0 ) {
        int selectedrow = pwadselectlist -> currentRow ();
        QListWidgetItem *selecteditem = pwadselectlist -> takeItem ( selectedrow );
        pwadselectlist -> insertItem ( selectedrow - 1 , selecteditem );
        pwadselectlist -> setCurrentRow ( selectedrow - 1 );
    }
}

void editprofile::buttonmovedown () {
    if ( pwadselectlist -> currentRow () < pwadselectlist -> count () - 1 ) {
        QListWidgetItem *selecteditem = pwadselectlist -> takeItem ( pwadselectlist -> currentRow () );
        pwadselectlist -> insertItem ( pwadselectlist -> currentRow () + 1 , selecteditem );
        pwadselectlist -> setCurrentRow ( pwadselectlist -> currentRow () + 1 );
    }
}

void editprofile::buttonremove () { pwadselectlist -> takeItem ( pwadselectlist -> row ( pwadselectlist -> currentItem () ) ); }

void editprofile :: buttonremoveiwad () {
	if ( iwadselect -> count () != 1 ) {	
		QMessageBox :: StandardButton ret = QMessageBox::question(this, "DoomRunner", "Are you sure you want to remove " + iwadselect -> currentText () + "? The file will be removed from \"./wads/iwads/" + iwadselect -> currentText () + ".wad, but it will remain in it's original location.", QMessageBox::Yes | QMessageBox::No );
   		if (ret == QMessageBox::Yes) {
			QString delfilename = iwadselect -> currentText ();
			iwadselect -> clear ();
			QString removedfile = QString :: fromStdString ( startdir ) + "/wads/iwads/" + delfilename + ".wad";
			remove ( removedfile . toStdString () . c_str () );
    		string dir = string( "." );
    		vector<string> files = vector<string>();
    		getdir ( startdir + "/wads/iwads", files );
    		size_t filesid;
    		string iwadlistitem;
    		for ( unsigned int i = 0; i < files.size(); i++ ) {
        		iwadlistitem = files[i];
        		transform(iwadlistitem.begin(), iwadlistitem.end(), iwadlistitem.begin(), ::tolower);
        		filesid = iwadlistitem.find(".wad");
				if (filesid != string::npos) {
            		transform(iwadlistitem.begin(), iwadlistitem.begin()+1, iwadlistitem.begin(), ::toupper);
            		iwadlistitem.erase(iwadlistitem.end()-4,iwadlistitem.end());
            		if ( iwadlistitem != "Prboom" ) { iwadselect -> addItem(QString::fromStdString(iwadlistitem)); }
        		}
 			}					
		}
	}
	else { QMessageBox :: information ( this, "DoomRunner" , "You must have at least one iwad set up with DoomRunner!" ); }
}
