/*-----------------------------------------------------------------------+
| Doom Runner: A cross-platform GUI frontend for Doom.                   |
| Copyright (C) 2011 Nick Virag                                          |
+------------------------------------------------------------------------+
| This program is free software: you can redistribute it and/or modify   |
| it under the terms of the GNU General Public License as published by   |
| the Free Software Foundation, either version 3 of the License, or      |
| any later version.                                                     |
+------------------------------------------------------------------------+
| This program is distributed in the hope that it will be useful,        |
| but WITHOUT ANY WARRANTY; without even the implied warranty of         |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
| GNU General Public License for more details.                           |
+------------------------------------------------------------------------+
| You should have received a copy of the GNU General Public License      |
| along with this program.  If not, see <http://www.gnu.org/licenses/>.  |
+------------------------------------------------------------------------+
| Contact Nick Virag at: <nick@win31.net>, <support@retronuggetdev.com>, |
| or <nickel516@gmail.com>.                                              |
+-----------------------------------------------------------------------*/
#include <QtGui/QtGui>
#include <iostream>
#include <fstream>
#include <string>
#include <cstring>
#include <sstream>
#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <vector>
#include "doomrunner.h"
#include "config.h"
#include "main.h"
#include "addnewfile.h"

QProcess * doomexecprocess;
int swapprocess;
QString engineoutput;

entrywindow :: entrywindow () {
    setFixedSize ( 551 , 296 );
    QToolBar * actionmenu = addToolBar ( "Program Menu" );
    actionmenu -> setFloatable ( false );
    actionmenu -> setMovable ( false );
    QWidget * spacer = new QWidget ();
    spacer -> setSizePolicy ( QSizePolicy :: Expanding , QSizePolicy :: Expanding );
    QAction * playaction = new QAction ( "&Play" , this );
    playaction -> setShortcut ( QKeySequence ( Qt :: Key_Return ) );
    actionmenu -> addAction ( playaction );
    QAction * helpaction = new QAction ( "&Help" , this );
    helpaction -> setShortcut ( tr ( "Ctrl+H" ) );
    actionmenu -> addAction ( helpaction );
    QAction * newaction = new QAction ( "&New" , this );
    newaction -> setShortcut( tr ( "Ctrl+N" ) );
    actionmenu -> addAction ( newaction );
    QAction * editaction = new QAction ( "&Edit" , this );
    editaction -> setShortcut ( tr ( "Ctrl+E" ) );
    actionmenu -> addAction ( editaction );
    QAction * eraseaction = new QAction ( "E&rase" , this );
    eraseaction -> setShortcut ( tr ( "Ctrl+X" ) );
    actionmenu -> addAction ( eraseaction );
    QAction * quitaction = new QAction ( "&Quit" , this );
    quitaction -> setShortcut ( tr ( "Ctrl+Q" ) );
    actionmenu -> addAction ( quitaction );
    actionmenu -> addWidget ( spacer );
    QAction * settingsaction = new QAction ( "&Settings" , this );
    if ( ostoggle2 == 2 ) { settingsaction -> setShortcut ( tr ( "Ctrl+," ) ); }
    else { settingsaction -> setShortcut( tr ( "Ctrl+P" ) ); }
    actionmenu -> addAction ( settingsaction );
    QAction * aboutaction = new QAction ( "&About" , this );
    aboutaction -> setShortcut ( tr ( "Ctrl+A" ) );
    actionmenu -> addAction ( aboutaction );
    gameselect = new QListWidget ( this );
    gameselect -> setSortingEnabled ( true );
    int profilevar = 0;
    bool addgameprofile = true;
    while ( addgameprofile == true ) {
        while ( preadf [ profilevar ] != "end" ) {
            QString profiletitle = QString :: fromStdString ( preadf [ profilevar ] );
            gameselect -> addItem ( new QListWidgetItem ( profiletitle ) );
            profilevar += 3;
        }
        addgameprofile = false;
    }
    gameimage = new QGraphicsView ( this );
    gamenotes = new QLabel ( this );
    gamenotes->setWordWrap ( true );
    gamenotes->setFrameShape ( QFrame :: StyledPanel );
    gamenotes->setAlignment ( Qt::AlignTop );
    buttonplay = new QPushButton( "Play" , this );
    buttonplay->setDefault( true );
    buttonhelp = new QPushButton( tr( "Help" ), this );
    buttonnew = new QPushButton( tr( "New" ), this );
    buttonedit = new QPushButton( tr( "Edit" ), this );
    buttondel = new QPushButton( tr( "Erase" ), this );
    buttonquit = new QPushButton( tr( "Quit" ), this );
    switch ( windowstyle ) {
        case 0:
            if ( ostoggle2 == 2 ) {
                if ( toolbartoggleint == 1 ) {
                    setFixedSize( 551, 270 );
                    setUnifiedTitleAndToolBarOnMac(true);
                    gameselect->setGeometry( 10, 10, 351, 201 );
                    gameimage->setGeometry( 370, 10, 171, 151 );
                    gamenotes->setGeometry( 370, 170, 171, 91 );
                    buttonplay->setGeometry( 4, 216, 114, 53 );
                    buttonhelp->setGeometry( 120, 220, 41, 41 );
                    buttonnew->setGeometry( 170, 220, 41, 41 );
                    buttonedit->setGeometry( 220, 220, 41, 41 );
                    buttondel->setGeometry( 270, 220, 41, 41 );
                    buttonquit->setGeometry( 320, 220, 41, 41 );
                }
                else {
                    gameselect->setGeometry( 10, 36, 351, 201 );
                    gameimage->setGeometry( 370, 36, 171, 151 );
                    gamenotes->setGeometry( 370, 196, 171, 91 );
                    buttonplay->setGeometry( 4, 242, 114, 53 );
                    buttonhelp->setGeometry( 120, 246, 41, 41 );
                    buttonnew->setGeometry( 170, 246, 41, 41 );
                    buttonedit->setGeometry( 220, 246, 41, 41 );
                    buttondel->setGeometry( 270, 246, 41, 41 );
                    buttonquit->setGeometry( 320, 246, 41, 41 );
                }
                break;
            }
        default:
            gameselect->setGeometry( 10, 36, 351, 201 );
            gameimage->setGeometry( 370, 36, 171, 151 );
            gamenotes->setGeometry( 370, 196, 171, 91 );
            buttonplay->setGeometry( 10, 246, 101, 41 );
            buttonhelp->setGeometry( 120, 246, 41, 41 );
            buttonnew->setGeometry( 170, 246, 41, 41 );
            buttonedit->setGeometry( 220, 246, 41, 41 );
            buttondel->setGeometry( 270, 246, 41, 41 );
            buttonquit->setGeometry( 320, 246, 41, 41 );
            break;
    }
    setWindowTitle( tr( "Doom Runner" ) );
    connect( buttonquit, SIGNAL( clicked() ), qApp, SLOT( quit() ) );
    connect( quitaction, SIGNAL( triggered() ), qApp, SLOT( quit() ) );
    connect( buttonplay, SIGNAL( clicked() ), this, SLOT( buttonplayf() ) );
    connect( playaction, SIGNAL( triggered() ), this, SLOT( buttonplayf() ) );
    connect( buttonhelp, SIGNAL( clicked() ), this, SLOT( buttonhelpf() ) );
    connect( helpaction, SIGNAL( triggered() ), this, SLOT( buttonhelpf() ) );
    connect( buttonnew, SIGNAL( clicked() ), this, SLOT( buttonnewf() ) );
    connect( newaction, SIGNAL( triggered() ), this, SLOT( buttonnewf() ) );
    connect( buttonedit, SIGNAL( clicked() ), this, SLOT( buttoneditf() ) );
    connect( editaction, SIGNAL( triggered() ), this, SLOT( buttoneditf() ) );
    connect( buttondel, SIGNAL( clicked() ), this, SLOT( buttondelf() ) );
    connect( eraseaction, SIGNAL( triggered() ), this, SLOT( buttondelf() ) );
    connect( settingsaction, SIGNAL( triggered() ), this, SLOT( settingsshow() ) );
    connect ( aboutaction, SIGNAL (triggered () ), this, SLOT ( aboutdoomrunner () ) );
    connect(gameselect, SIGNAL( currentItemChanged( QListWidgetItem *, QListWidgetItem * ) ), this, SLOT( updateMenus() ) );
    connect(gameselect, SIGNAL( itemDoubleClicked( QListWidgetItem * ) ), this, SLOT( buttonplayf() ) );
    gameselect->setCurrentRow( currentselected );
    entrywindow::updateMenus();
    show();
    if ( sliderint == 1 ) { for ( int i = -270; i < QApplication::desktop()->width() / 2 - 275; i += sliderspeedint ) { move( i, QApplication::desktop()->height() / 2 - 135 ); } }
    else {
        if ( windowx != 0 ) { move ( windowx, windowy ); }
    }
}

void entrywindow::buttonplayf() {
    string type, executable, glexecutable, line, iwad, pwad1, pwad2, pwad3, pwad4, deh1, deh2, deh3, deh4, musictoggle, soundtoggle, opengltoggle, widthres, heightres, mousetoggle, joysticktoggle, addcmd;
    StringList progpath;
    string progexecloc;
    string settingsdir = startdir + "/profiles/profile" + currentmenuitem + ".txt";
    ifstream settingsfile ( settingsdir.c_str() );
    if ( settingsfile.is_open() ) {
        string tinput;
        while ( ! settingsfile.eof() ) {
            getline ( settingsfile, line );
            tinput = tinput + "^" + line;
	}
        tinput.erase( 0, 1 );
        string::size_type parser = tinput.find_first_of( "^" );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        type = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        executable = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        glexecutable = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        iwad = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        pwad1 = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        pwad2 = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        pwad3 = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        pwad4 = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        deh1 = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        deh2 = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        deh3 = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        deh4 = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        musictoggle = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        soundtoggle = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        opengltoggle = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        widthres = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        heightres = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        mousetoggle = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        joysticktoggle = tinput.substr( 0, parser );
        tinput.erase( 0, parser + 1 );
        parser = tinput.find_first_of( "^" );
        addcmd = tinput.substr( 0, parser );
        settingsfile.close();
    }
    progpath << "-iwad" << QString :: fromStdString ( startdir + "/wads/iwads/" + iwad + ".wad" );
    if ( type == "def-prboom" ) {
        if ( ostoggle2 == 2 ) {	progexecloc = startdir + "/PrBoom"; }
        else if (ostoggle2 == 1) { progexecloc =  startdir + "/wads/iwads/PrBoom.exe";	}
        else {
			if ( globaltoggle == 1 ) { progexecloc = startdir + "/prboom"; }
			else { progexecloc = "prboom"; }
		}
        if (opengltoggle == "true") {
            if ( ostoggle2 == 1 ) { progexecloc =  startdir + "/wads/iwads/glboom.exe"; }
            else { progpath << "-vidmode" << "gl"; }
        }
    }
    else if ( type == "prboom" || type == "prboom=gl" || type == "zdoom" || type == "zdoom=gl" || type == "other" ) {
        if ( type == "prboom" || type == "zdoom" || type == "other" ) { progexecloc = executable; }
        else if ( type == "prboom=gl" || type == "zdoom=gl" ) { progexecloc = glexecutable; }
    }
    else if ( type == "prboom+=gl" || type == "zdoom+=gl" || type == "other+=gl" ) {
        if (opengltoggle == "true") { progexecloc = glexecutable; }
        else if (opengltoggle != "true") { progexecloc = executable; }
    }
    else if ( type == "legacy" ) {
        progexecloc = executable;
        //progpath = "\"" + executable + "\" -iwad \"" + startdir + "/wads/iwads/" + iwad + ".wad\"";
        if (opengltoggle == "true") {
			if ( glestoggle == 0 ) { progpath << "-opengl"; }
			else { progpath << "-opengl" << "OpenGL-ES"; }
		}
    }
    if (pwad1 != "<none>") { progpath << "-file" << QString :: fromStdString ( startdir + "/wads/" + pwad1 + ".wad" ); }
    if (pwad2 != "<none>") { progpath << QString :: fromStdString ( startdir + "/wads/" + pwad2 + ".wad" ); }
    if (pwad3 != "<none>") { progpath << QString :: fromStdString ( startdir + "/wads/" + pwad3 + ".wad"); }
    if (pwad4 != "<none>") { progpath << QString :: fromStdString ( startdir + "/wads/" + pwad4 + ".wad"); }
    string linef , wadfileoutput , dehfileoutput , tinputf [ 64 ] , tinputd [ 64 ];
	tinputd [ 0 ] = "";
	int tinputdloc = 0;
	size_t filesid;
    string wadfiledir = startdir + "/profiles/profile" + currentmenuitem + "f.txt";
    ifstream wadfileloc ( wadfiledir . c_str () );
    if ( wadfileloc.is_open () ) {
		progpath << "-file";
        for ( int inputloc = 0 ; ! wadfileloc.eof () ; inputloc ++ ) {
            if (  inputloc != 65 ) {
                getline ( wadfileloc , linef );
                if ( linef != "" ) {
                    tinputf [ inputloc ] = linef;
                    if ( tinputf [ inputloc ] . substr ( 0 , 1 ) == "c" ) {
                        tinputf [ inputloc ] . erase ( 0, 1 );
						filesid = tinputf [ inputloc ] . find ( ".wad" );
						if ( filesid != string :: npos ) { progpath << QString :: fromStdString ( tinputf [ inputloc ] ); }
						else {
							if ( type == "legacy" ) { progpath << QString :: fromStdString ( tinputf [ inputloc ] ); }
							else {
								tinputd [ tinputdloc ] = tinputf [ inputloc ];
								tinputdloc++;
							}
						}
                    }
                }
            }
        }
		if ( tinputd [ 0 ] != "" ) {
			progpath << "-deh";			
			for ( int x = 0; tinputd [ x ] != ""; x++ ) {
				progpath << QString :: fromStdString ( tinputd [ x ] );
			}
		}
    }
    wadfileloc . close ();
    if (type == "legacy") {if (deh1 != "<none>") { progpath << QString :: fromStdString ( startdir + "/wads/deh/" + deh1 + ".deh" ); } }
    else {if (deh1 != "<none>") { progpath << "-deh" << QString :: fromStdString ( startdir + "/wads/deh/" + deh1 + ".deh" ); } }
    if (deh2 != "<none>") { progpath << QString :: fromStdString ( startdir + "/wads/deh/" + deh2 + ".deh" ); }
    if (deh3 != "<none>") { progpath << QString :: fromStdString ( startdir + "/wads/deh/" + deh3 + ".deh" ); }
    if (deh4 != "<none>") { progpath << QString :: fromStdString ( startdir + "/wads/deh/" + deh4 + ".deh" ); }
    if (musictoggle == "<nomusic>") { progpath << "-nomusic"; }
    if (soundtoggle == "<nosound>") { progpath << "-nosound"; }
    if (mousetoggle == "<nomouse>") { progpath << "-nomouse"; }
    if (joysticktoggle == "<nojoystick>") { progpath << "-nojoy"; }
    if (addcmd != "<none>") { progpath << QString :: fromStdString ( addcmd ); }
    progpath << "-width" << QString :: fromStdString ( widthres ) << "-height" << QString :: fromStdString ( heightres );
	doomexecprocess = new QProcess ( this );
	QStringList commandline;
	connect ( doomexecprocess , SIGNAL ( readyReadStandardOutput () ) , this , SLOT ( appendTextBox () ) );
	connect ( doomexecprocess , SIGNAL ( finished ( int , QProcess :: ExitStatus ) ) , this , SLOT ( doomenginefinished () ) );
	if ( swapprocess != 2 ) { swapprocess = 3; }
	if ( swapprocess == 3 ) {
		if ( askruntoggle == 1 ) {
			QMessageBox :: StandardButton qvalue;
			qvalue = QMessageBox :: question ( this , "Run this command?", QString :: fromStdString ( progexecloc ) + " " + ( QString ) progpath , QMessageBox :: Yes | QMessageBox :: No );
            if ( qvalue == QMessageBox :: Yes ) {
				buttonplay -> setText ( "Running..." );
				buttonplay -> setEnabled ( false );
				swapprocess = 2;
				doomexecprocess -> start ( QString :: fromStdString ( progexecloc ) , progpath ); }
			}
		else {
			buttonplay -> setText ( "Running..." );
			buttonplay -> setEnabled ( false );
			swapprocess = 2;
			doomexecprocess -> start ( QString :: fromStdString ( progexecloc ) , progpath );
		}
	}
}

void entrywindow::buttonhelpf() { QMessageBox::information(this, "Doom Runner Help", "<b>Play</b>: Play the selected game, using a Doom port such as PrBoom.<br><b>Help</b>: Show this window.<br><b>New</b>: Create a new \"profile\", a game configuration for Doom.<br><b>Edit</b>: Change the settings in the currently selected Doom game.<br><b>Erase</b>: Delete a DoomRunner game profile. No save files or special Doom files such as wads are deleted.<br><b>Quit</b>: Exit DoomRunner.<br><b>Settings</b>: Access DoomRunner-specific settings that aren't a part of Doom itself.<br><b>About</b>: Get licensing and contact information."); }

void entrywindow::buttoneditf() {
    displayswitch = 3;
    QString selecteditem = gameselect->currentItem()->text();
    currentselected = gameselect->currentRow();
    int profilevar = 0;
    while (preadf[profilevar] != "") {
        QString profiletitle = QString::fromStdString(preadf[profilevar]);
        if (profiletitle == selecteditem) {
            titletransfer = preadf[profilevar];
            notestransfer = preadf[profilevar + 1];
            imagetransfer = preadf[profilevar + 2];
        }
        profilevar += 3;
    }
    if ( sliderint == 1 ) { for ( int i = QApplication::desktop()->width() / 2 - 275; i > -551; i -= sliderspeedint ) { move( i, QApplication::desktop()->height() / 2 - 135 ); } }
    else {
        windowx = x();
        windowy = y();
    }
    this->close();
    editprofile *widget = new editprofile;
    widget->show();
    if ( selectedtab == 4 ) { selectedtab = 0; }
}

int getdir1 (string dir, vector<string> &files) {
    DIR *dp;
    struct dirent *dirp;
    if((dp  = opendir(dir.c_str())) == NULL) {
        cout << "Error(" << errno << ") opening " << dir << endl;
        return errno;
    }
    while ((dirp = readdir(dp)) != NULL) {
        files.push_back(string(dirp->d_name));
    }
    closedir(dp);
    return 0;
}

void entrywindow::buttonnewf() {
    if ( sliderint == 1 ) { for ( int i = QApplication::desktop()->width() / 2 - 275; i > -551; i -= sliderspeedint ) { move( i, QApplication::desktop()->height() / 2 - 135 ); } }
    else {
        windowx = x();
        windowy = y();
    }
    displayswitch = 3;
    addnewfile *namewidget = new addnewfile;
    namewidget->show();
    entrywindow::programclose();
}

void entrywindow::buttondelf() {
    bool continuetoggle = true;
    QString selecteditem = gameselect->currentItem()->text();
    int profilevar = 0;
    while (preadf[profilevar] != "") {
        QString profiletitle = QString::fromStdString(preadf[profilevar]);
        if (profiletitle == selecteditem) {
            selecteditem += 1;
            currentselecteditem = profilevar / 3 + 1;
        }
        profilevar += 3;
    }
    int curritem = currentselecteditem;
    if (curritem != -1) {
        QMessageBox::StandardButton ret;
        ret = QMessageBox::question(this, "Doom Runner", "Are you sure you want to delete " + gameselect->currentItem()->text() + "? This will only remove the profile from Doom Runner; no game files will be removed.", QMessageBox::Yes | QMessageBox::No);
        if (ret == QMessageBox::Yes) {
            string removedir = startdir + "/profiles/profile" + currentmenuitem + ".txt";
            if ( removedir == startdir + "/profiles/profile1.txt" ) {
                string checkprofile2 = startdir + "/profiles/profile2.txt";
                ifstream checkprofile( checkprofile2.c_str() );
                if ( checkprofile ) {
                    continuetoggle = true; 
                    checkprofile.close();
                }
                else {
                    continuetoggle = false;
                    QMessageBox::critical( this, "Doom Runner", "DoomRunner requires at least one profile to exist. This one can't be deleted until another one is created." );
                }
            }
            else { continuetoggle = true; }
            if ( continuetoggle == true ) {
                remove( removedir.c_str () );
				string removedir2 = startdir + "/profiles/profile" + currentmenuitem + "f.txt";
				remove ( removedir2.c_str () );
                int removedprofile = atoi ( currentmenuitem.c_str () ) + 1;
                int newprofile;
                stringstream currentmenuitemtemp;
                stringstream newprofiletemp;
                bool whiletoggle = true;
                gameselect->takeItem ( gameselect->currentRow () );
                while ( whiletoggle == true ) {
                    currentmenuitemtemp.str ( std::string () );
                    currentmenuitemtemp << removedprofile - 1;
                    newprofile = removedprofile;
                    newprofiletemp.str ( std::string () );
                    newprofiletemp << newprofile;
                    string newfile = startdir + "/profiles/profile" + currentmenuitemtemp.str () + ".txt";
                    string openfile = startdir + "/profiles/profile" + newprofiletemp.str () + ".txt";
                    ifstream settingsfile ( openfile.c_str () );
                    if ( settingsfile ) {
                        settingsfile.close();
                        rename ( openfile.c_str (), newfile.c_str () );
                        removedprofile += 1;
                    }
                    else { whiletoggle = false; }
                }
            }
        close ();
        interfaceviews ( false );
        entrywindow *widget = new entrywindow;
        widget->show ();
        }
    }
}

void entrywindow::updateMenus() {
    QString imagedata;
    int currentmenuitemint = 0;
    stringstream currentmenuitemtemp;
    if ( preadf[0] == "" ) {
        QMessageBox::information ( this, "Doom Runner", "This is your first running Doom Runner. You need to set up a profile before playing Doom.\nYou will be promted for some of the basic options.\nAddititonal settings can be accessed later using the Edit button." );
        imagedata = QString::fromStdString(startdir) + "/profiles/default.png";
        loadimage = new QGraphicsScene(gameimage);
        loadimage->setSceneRect( gameimage->rect() );
        gameimage->setScene(loadimage);
        QPixmap imagefile(imagedata);
        gameimage->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        gameimage->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        loadimage->addPixmap(imagefile);
        entrywindow::buttonnewf();
    }
    else {
    QString selecteditem = gameselect->currentItem()->text();
    int profilevar = 0;
    string profilenotes;
    string profileimage;
    while (preadf[profilevar] != "") {
        QString profiletitle = QString::fromStdString(preadf[profilevar]);
        if (profiletitle == selecteditem) {
            profilenotes = preadf[profilevar + 1];
            profileimage = preadf[profilevar + 2];
            gamenotes->setText(QString::fromStdString(profilenotes));
            imagedata = QString::fromStdString(profileimage);
            selecteditem += 1;
            currentmenuitemint ++;
            currentselecteditem = profilevar / 3 + 1;
        }
        profilevar += 3;
    }
    currentmenuitemtemp << currentselecteditem;
    currentmenuitem = currentmenuitemtemp.str();
    if (imagedata == "default") imagedata = QString::fromStdString(startdir) + "/profiles/default.png";
    loadimage = new QGraphicsScene(gameimage);
    loadimage->setSceneRect( gameimage->rect() );
    gameimage->setScene(loadimage);
    QPixmap imagefile(imagedata);
    gameimage->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    gameimage->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    QSize scaledSize = imagefile.size();
    scaledSize.scale(gameimage->size(), Qt::KeepAspectRatio);
    loadimage->addPixmap(imagefile.scaled(gameimage->size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
    }
}

void entrywindow::programclose() { this->close(); }

void entrywindow::settingsshow() {
    selectedtab = 4;
    entrywindow::buttoneditf();
}

void entrywindow :: appendTextBox () {
    QByteArray newData;
    newData = doomexecprocess -> readAllStandardOutput ();
	engineoutput += QString :: fromLocal8Bit ( newData );
}

void entrywindow::aboutdoomrunner () { QMessageBox::information(this, "About Doom Runner 1.0.3", "Doom Runner 1.0.3 Copyright (C) 2011 Nicholas Virag\nThis program comes with ABSOLUTELY NO WARRANTY; This is free software, and you are welcome to redistribute it under certain conditions; for details see the included \"./gpl-3.0.txt\".\nContact: support@retronuggetdev.com\nWebsite: http://www.retronuggetdev.com" ); }

void entrywindow :: doomenginefinished () {
	if ( askruntoggle == 1 ) { QMessageBox :: information ( this , "Engine Output" , engineoutput ); }
	buttonplay -> setEnabled ( true );
	buttonplay -> setText ( "Play" );
	swapprocess = 3;
}
